"""
Python module generated from Java source file dev.magicmq.pyspigot.PluginListener

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot import *
from dev.magicmq.pyspigot.util import StringUtils
from dev.magicmq.pyspigot.util.player import PlayerAdapter
from net.md_5.bungee.api import ChatColor
from net.md_5.bungee.api.chat import BaseComponent
from net.md_5.bungee.api.chat import ClickEvent
from net.md_5.bungee.api.chat import ComponentBuilder
from net.md_5.bungee.api.chat import HoverEvent
from net.md_5.bungee.api.chat import TextComponent
from typing import Any, Callable, Iterable, Tuple


class PluginListener:
    """
    The primary listener for PySpigot. Methods called in this class are called via platform-specific listeners.
    """





"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.libraries.JarClassLoader

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.manager.libraries import *
from java.net import MalformedURLException
from java.net import URL
from java.net import URLClassLoader
from java.nio.file import Path
from typing import Any, Callable, Iterable, Tuple


class JarClassLoader(URLClassLoader):
    """
    Utility class for assisting with loading Jar files into the classpath.
    """

    def __init__(self, parentClassLoader: "ClassLoader"):
        """
        Initialize a new JarClassLoader using a parent class loader.

        Arguments
        - parentClassLoader: The parent class loader to use
        """
        ...


    def addJarToClasspath(self, file: "Path") -> None:
        """
        Add a new Jar to the classpath.

        Arguments
        - file: The Jar file to add to the classpath

        Raises
        - MalformedURLException: If the file has an invalid URL
        """
        ...


    def isJarInClassPath(self, file: "Path") -> bool:
        """
        Check if a Jar file is in the classpath.

        Arguments
        - file: The Jar file to check

        Returns
        - True if the Jar file is already in the classpath, False if otherwise
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.manager.script.BukkitScript

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit.manager.script import *
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.manager.script import ScriptOptions
from java.nio.file import Path
from org.bukkit import Bukkit
from org.bukkit.permissions import Permission
from typing import Any, Callable, Iterable, Tuple


class BukkitScript(Script):
    """
    An extension of the base Script class that includes Bukkit-specific code for initializing script permissions.
    """

    def __init__(self, path: "Path", name: str, options: "BukkitScriptOptions"):
        """
        Arguments
        - path: The path that corresponds to the file where the script lives
        - name: The name of this script. Should contain its extension (.py)
        - options: The ScriptOptions for this script
        """
        ...


    def initPermissions(self) -> None:
        """
        Adds the script's permission (from its options) to the server.
        """
        ...


    def removePermissions(self) -> None:
        """
        Removes the script's permissions from the server.
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.manager.protocol.PacketReceivingListener

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.comphenix.protocol import PacketType
from com.comphenix.protocol.events import ListenerPriority
from com.comphenix.protocol.events import PacketEvent
from dev.magicmq.pyspigot.bukkit.manager.protocol import *
from dev.magicmq.pyspigot.manager.script import Script
from org.python.core import PyFunction
from typing import Any, Callable, Iterable, Tuple


class PacketReceivingListener(ScriptPacketListener):
    """
    A listener that listens for packets received by the server from the client.

    See
    - ScriptPacketListener
    """

    def __init__(self, script: "Script", function: Callable, packetType: "PacketType", listenerPriority: "ListenerPriority", listenerType: "ListenerType"):
        """
        Arguments
        - script: The script associated with this packet listener
        - function: The function to be called when the packet is received
        - packetType: The packet type to listen for
        - listenerPriority: The com.comphenix.protocol.events.ListenerPriority of this listener
        - listenerType: The ListenerType of this listener
        """
        ...


    def onPacketReceiving(self, event: "PacketEvent") -> None:
        """

        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.manager.placeholder.PlaceholderManager

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit.manager.placeholder import *
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.util import ScriptUtils
from java.util import Objects
from org.python.core import PyFunction
from typing import Any, Callable, Iterable, Tuple


class PlaceholderManager:
    """
    Manager to interface with PlaceholderAPI. Primarily used by scripts to register and unregister placeholder expansions.
    
    Do not call this manager if PlaceholderAPI is not loaded and enabled on the server! It will not work.
    """

    def registerPlaceholder(self, placeholderFunction: Callable) -> "ScriptPlaceholder":
        """
        Register a new script placeholder expansion.
        
        If you want to register a relational placeholder expansion, see .registerPlaceholder(PyFunction, PyFunction)
        
        **Note:** This should be called from scripts only!

        Arguments
        - placeholderFunction: The function that should be called when the placeholder is used

        Returns
        - A ScriptPlaceholder representing the placeholder expansion that was registered
        """
        ...


    def registerPlaceholder(self, placeholderFunction: Callable, relPlaceholderFunction: Callable) -> "ScriptPlaceholder":
        """
        Register a new script placeholder expansion, including relational placeholders.
        
        **Note:** This should be called from scripts only!

        Arguments
        - placeholderFunction: The function that should be called when the placeholder is used
        - relPlaceholderFunction: The function that should be called when the relational placeholder

        Returns
        - A ScriptPlaceholder representing the placeholder expansion that was registered
        """
        ...


    def registerPlaceholder(self, placeholderFunction: Callable, author: str, version: str) -> "ScriptPlaceholder":
        """
        Register a new script placeholder expansion.
        
        If you want to register a relational placeholder expansion, see .registerPlaceholder(PyFunction, PyFunction, String, String)
        
        **Note:** This should be called from scripts only!

        Arguments
        - placeholderFunction: The function that should be called when the placeholder is used
        - author: The author of the placeholder
        - version: The version of the placeholder

        Returns
        - A ScriptPlaceholder representing the placeholder expansion that was registered
        """
        ...


    def registerPlaceholder(self, placeholderFunction: Callable, relPlaceholderFunction: Callable, author: str, version: str) -> "ScriptPlaceholder":
        """
        Register a new script placeholder expansion, including relational placeholders.
        
        **Note:** This should be called from scripts only!

        Arguments
        - placeholderFunction: The function that should be called when the placeholder is used
        - relPlaceholderFunction: The function that should be called when the relational placeholder
        - author: The author of the placeholder
        - version: The version of the placeholder

        Returns
        - A ScriptPlaceholder representing the placeholder expansion that was registered
        """
        ...


    def unregisterPlaceholder(self, placeholder: "ScriptPlaceholder") -> None:
        """
        Unregister a script placeholder expansion.
        
        **Note:** This should be called from scripts only!

        Arguments
        - placeholder: The placeholder expansion to unregister
        """
        ...


    def unregisterPlaceholder(self, script: "Script") -> bool:
        """
        Unregister a script's placeholder expansion.
        
        Similar to .unregisterPlaceholder(ScriptPlaceholder), except this method can be called from outside a script to unregister a script's placeholder expansion (for example when the script is unloaded and stopped).

        Arguments
        - script: The script whose placeholder should be unregistered

        Returns
        - True if a placeholder was unregistered, False if the script did not have a placeholder registered previously
        """
        ...


    def getPlaceholder(self, script: "Script") -> "ScriptPlaceholder":
        """
        Get a script's placeholder expansion.

        Arguments
        - script: The script to get the placeholder expansion from

        Returns
        - The script's placeholder expansion, null if the script does not have a placeholder expansion registered
        """
        ...


    @staticmethod
    def get() -> "PlaceholderManager":
        """
        Get the singleton instance of this PlaceholderManager.

        Returns
        - The instance
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.command.PySpigotCommand

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot import PyCore
from dev.magicmq.pyspigot.command import *
from dev.magicmq.pyspigot.command.subcommands import HelpCommand
from dev.magicmq.pyspigot.command.subcommands import InfoCommand
from dev.magicmq.pyspigot.command.subcommands import ListScriptsCommand
from dev.magicmq.pyspigot.command.subcommands import LoadCommand
from dev.magicmq.pyspigot.command.subcommands import LoadLibraryCommand
from dev.magicmq.pyspigot.command.subcommands import ReloadAllCommand
from dev.magicmq.pyspigot.command.subcommands import ReloadCommand
from dev.magicmq.pyspigot.command.subcommands import ReloadConfigCommand
from dev.magicmq.pyspigot.command.subcommands import UnloadCommand
from dev.magicmq.pyspigot.util.player import CommandSenderAdapter
from java.util import Arrays
from net.md_5.bungee.api import ChatColor
from typing import Any, Callable, Iterable, Tuple


class PySpigotCommand:

    def __init__(self):
        ...


    def onCommand(self, sender: "CommandSenderAdapter", label: str, args: list[str]) -> bool:
        ...


    def onTabComplete(self, sender: "CommandSenderAdapter", args: list[str]) -> list[str]:
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.util.ReflectionUtils

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit.util import *
from java.lang.reflect import Field
from java.lang.reflect import Method
from org.bukkit import Bukkit
from typing import Any, Callable, Iterable, Tuple


class ReflectionUtils:
    """
    A utility class to simplify reflection for working with CraftBukkit and NMS classes.
    """

    @staticmethod
    def getNMSClass(packageName: str, className: str) -> type[Any]:
        ...


    @staticmethod
    def getCraftBukkitClass(className: str) -> type[Any]:
        ...


    @staticmethod
    def getCraftBukkitClass(packageName: str, className: str) -> type[Any]:
        ...


    @staticmethod
    def getMethod(clazz: type[Any], methodName: str) -> "Method":
        ...


    @staticmethod
    def getField(clazz: type[Any], fieldName: str) -> "Field":
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.event.ScriptUnloadEvent

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit.event import *
from dev.magicmq.pyspigot.manager.script import Script
from org.bukkit.event import HandlerList
from typing import Any, Callable, Iterable, Tuple


class ScriptUnloadEvent(ScriptEvent):
    """
    Called when a script is unloaded.
    """

    def __init__(self, script: "Script", error: bool):
        """
        Arguments
        - script: The script that was unloaded
        - error: Whether the script was unloaded due to an error
        """
        ...


    def isError(self) -> bool:
        """
        Get if this unload event was due to a script error.

        Returns
        - True if the script was unloaded due to an error/exception, False if otherwise
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.config.BukkitPluginConfig

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit import PySpigot
from dev.magicmq.pyspigot.bukkit.config import *
from dev.magicmq.pyspigot.config import PluginConfig
from java.time.format import DateTimeFormatter
from java.util import Properties
from org.bukkit.configuration.file import FileConfiguration
from typing import Any, Callable, Iterable, Tuple


class BukkitPluginConfig(PluginConfig):
    """
    The Bukkit-specific implementation of the dev.magicmq.pyspigot.config.PluginConfig class, for retreiving values from the plugin config.yml.
    """

    def reload(self) -> None:
        ...


    def getMetricsEnabled(self) -> bool:
        ...


    def getScriptLoadDelay(self) -> int:
        ...


    def getLibraryRelocations(self) -> dict[str, str]:
        ...


    def getLogTimestamp(self) -> "DateTimeFormatter":
        ...


    def doScriptActionLogging(self) -> bool:
        ...


    def doVerboseRedisLogging(self) -> bool:
        ...


    def doScriptUnloadOnPluginDisable(self) -> bool:
        ...


    def scriptOptionEnabled(self) -> bool:
        ...


    def scriptOptionLoadPriority(self) -> int:
        ...


    def scriptOptionPluginDepend(self) -> list[str]:
        ...


    def scriptOptionFileLoggingEnabled(self) -> bool:
        ...


    def scriptOptionMinLoggingLevel(self) -> str:
        ...


    def scriptOptionPermissionDefault(self) -> str:
        ...


    def scriptOptionPermissions(self) -> dict[Any, Any]:
        ...


    def shouldPrintStackTraces(self) -> bool:
        ...


    def shouldShowUpdateMessages(self) -> bool:
        ...


    def loadJythonOnStartup(self) -> bool:
        ...


    def getJythonProperties(self) -> "Properties":
        ...


    def getJythonArgs(self) -> list[str]:
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.manager.command.BukkitCommandManager

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot import PyCore
from dev.magicmq.pyspigot.bukkit.manager.command import *
from dev.magicmq.pyspigot.bukkit.util import ReflectionUtils
from dev.magicmq.pyspigot.manager.command import CommandManager
from dev.magicmq.pyspigot.manager.command import ScriptCommand
from dev.magicmq.pyspigot.manager.script import Script
from java.lang.reflect import Field
from java.lang.reflect import InvocationTargetException
from java.lang.reflect import Method
from org.bukkit import Bukkit
from org.bukkit.command import Command
from org.bukkit.command import SimpleCommandMap
from org.python.core import PyFunction
from typing import Any, Callable, Iterable, Tuple


class BukkitCommandManager(CommandManager):
    """
    The Bukkit-specific implementation of the command manager.
    """

    @staticmethod
    def get() -> "BukkitCommandManager":
        """
        Get the singleton instance of this BukkitCommandManager.

        Returns
        - The instance
        """
        ...



from .ScriptPacketListener import ScriptPacketListener
from .ProtocolManager import ProtocolManager
from .PacketSendingListener import PacketSendingListener
from .PacketReceivingListener import PacketReceivingListener
from .ListenerType import ListenerType
from .AsyncProtocolManager import AsyncProtocolManager


"""
Python module generated from Java source file dev.magicmq.pyspigot.util.logging.PrintStreamWrapper

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.util.logging import *
from java.io import OutputStream
from java.io import PrintStream
from java.nio.charset import StandardCharsets
from java.util import Arrays
from typing import Any, Callable, Iterable, Tuple


class PrintStreamWrapper(PrintStream):
    """
    A wrapper class that captures print statements and errors/exceptions from scripts and redirects them to the script's logger.
    """

    def __init__(self, out: "OutputStream", script: "Script", level: "Level", prefix: str):
        """
        Arguments
        - out: The parent OutputStream, usually System.out or System.err
        - script: The script to which this PrintStreamWrapper belongs
        - level: The logging level, usually Level.INFO for stdout and Level.SEVERE for stderr
        - prefix: The prefix to include before the log message, usually [STDOUT] for stdout and [STDERR] for stderr
        """
        ...


    def write(self, buf: list[int], off: int, len: int) -> None:
        """
        Captures writes to the PrintStream, converts the bytes into readable text (truncating according to the specified length and offset), and logs the text to the script's logger. This method also strips carriage returns/new line characters from the end of the text, because the script logger already inserts a new line when logging.

        Arguments
        - buf: A byte array
        - off: Offset from which to start taking bytes
        - len: Number of bytes to write
        """
        ...


    def write(self, buf: list[int]) -> None:
        """

        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.PySpigot

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot import PlatformAdapter
from dev.magicmq.pyspigot import PyCore
from dev.magicmq.pyspigot.bukkit import *
from dev.magicmq.pyspigot.bukkit.command import BukkitPluginCommand
from dev.magicmq.pyspigot.bukkit.config import BukkitPluginConfig
from dev.magicmq.pyspigot.bukkit.manager.command import BukkitCommandManager
from dev.magicmq.pyspigot.bukkit.manager.config import BukkitConfigManager
from dev.magicmq.pyspigot.bukkit.manager.listener import BukkitListenerManager
from dev.magicmq.pyspigot.bukkit.manager.placeholder import PlaceholderManager
from dev.magicmq.pyspigot.bukkit.manager.protocol import ProtocolManager
from dev.magicmq.pyspigot.bukkit.manager.script import BukkitScriptManager
from dev.magicmq.pyspigot.bukkit.manager.task import BukkitTaskManager
from dev.magicmq.pyspigot.config import PluginConfig
from dev.magicmq.pyspigot.manager.script import ScriptManager
from java.nio.file import Path
from java.nio.file import Paths
from org.bstats.bukkit import Metrics
from org.bstats.charts import SimplePie
from org.bukkit import Bukkit
from org.bukkit.command import PluginCommand
from org.bukkit.command import SimpleCommandMap
from org.bukkit.help import IndexHelpTopic
from org.bukkit.plugin import Plugin
from org.bukkit.plugin.java import JavaPlugin
from org.bukkit.scheduler import BukkitTask
from typing import Any, Callable, Iterable, Tuple


class PySpigot(JavaPlugin, PlatformAdapter):
    """
    Entry point of PySpigot for Bukkit servers.
    """

    def onEnable(self) -> None:
        ...


    def onDisable(self) -> None:
        ...


    def initConfig(self) -> "PluginConfig":
        ...


    def initCommands(self) -> None:
        ...


    def initListeners(self) -> None:
        ...


    def initPlatformManagers(self) -> None:
        ...


    def initVersionChecking(self) -> None:
        ...


    def setupMetrics(self) -> None:
        ...


    def shutdownMetrics(self) -> None:
        ...


    def shutdownVersionChecking(self) -> None:
        ...


    def getDataFolderPath(self) -> "Path":
        ...


    def getPluginClassLoader(self) -> "ClassLoader":
        ...


    def getVersion(self) -> str:
        ...


    def getPluginIdentifier(self) -> str:
        ...


    def isProtocolLibAvailable(self) -> bool:
        """
        Check if ProtocolLib is available on the server.

        Returns
        - True if ProtocolLib is loaded and enabled, False if otherwise
        """
        ...


    def isPlaceholderApiAvailable(self) -> bool:
        """
        Check if PlacehodlerAPI is available on the server.

        Returns
        - True if PlaceholderAPI is loaded and enabled, False if otherwise
        """
        ...


    @staticmethod
    def get() -> "PySpigot":
        """
        Get the instance of this plugin.

        Returns
        - The instance
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.script.Script

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot import PyCore
from dev.magicmq.pyspigot.manager.script import *
from dev.magicmq.pyspigot.util import ScriptUtils
from dev.magicmq.pyspigot.util.logging import PrintStreamWrapper
from dev.magicmq.pyspigot.util.logging import ScriptLogger
from java.io import File
from java.io import IOException
from java.nio.file import Path
from org.python.util import PythonInterpreter
from typing import Any, Callable, Iterable, Tuple


class Script(Comparable):
    """
    An object that represents a loaded script. Because this object is instantiated some time before the script is actually executed (in order to fetch its options and order scripts to load according to dependencies), there may be a brief time when this object represents a loaded *but not running * script. To check if this script object represents a running script, call ScriptManager.isScriptRunning(String).
    """

    def __init__(self, path: "Path", name: str, options: "ScriptOptions"):
        """
        Arguments
        - path: The path that corresponds to the file where the script lives
        - name: The name of this script. Should contain its extension (.py)
        - options: The ScriptOptions for this script
        """
        ...


    def close(self) -> None:
        """
        Closes this script's file logger and interpreter. Called when a script is unloaded/stopped.
        """
        ...


    def getFile(self) -> "File":
        """
        Get the File associated with this script.

        Returns
        - The File associated with this script
        """
        ...


    def getPath(self) -> "Path":
        """
        Get the path corresponding to the script file.

        Returns
        - The path
        """
        ...


    def getName(self) -> str:
        """
        Get the name associated with this script.

        Returns
        - The name associated with this script. Will contain its extension (.py)
        """
        ...


    def getSimpleName(self) -> str:
        """
        Get the simple name (without the file extension, .py) associated with this script.

        Returns
        - The simple name associated with this script. Will contain only the file name, without the extension (.py)
        """
        ...


    def getOptions(self) -> "ScriptOptions":
        """
        Get the ScriptOptions for this script, which contains various runtime options associated with this script.

        Returns
        - The ScriptOptions for this script
        """
        ...


    def getInterpreter(self) -> "PythonInterpreter":
        """
        Get the org.python.util.PythonInterpreter associated wtih this script.

        Returns
        - The org.python.util.PythonInterpreter associated with this script
        """
        ...


    def getLogger(self) -> "ScriptLogger":
        """
        Get this scripts logger.

        Returns
        - This script's logger

        See
        - ScriptLogger
        """
        ...


    def getLogFileName(self) -> str:
        """
        Get the log file name for this script.

        Returns
        - The log file name for this script. Will contain its extension (.log)
        """
        ...


    def getUptime(self) -> int:
        """
        Get the millisecond duration that this script has been loaded

        Returns
        - The duration that the script has been loaded
        """
        ...


    def compareTo(self, other: "Script") -> int:
        """
        Compares this script to another script, using load order as the primary comparison. If the load order of this script is higher than other, then this script will be considered "less" than other (I.E. sorted earlier in a set than the other script). If the load order of this script is lower than other, then this script will be considered "greater" than the other script (I.E. sorted later in a set than other).
        
        If the load priority of this script is equal to other, then a comparison is performed based on the name of the two scripts (alphabetical).

        Arguments
        - other: The other script to be compared

        Returns
        - 1 if this script is greater than other, -1 if this script is less than other, and 0 if the two scripts are equal
        """
        ...


    def equals(self, other: "Object") -> bool:
        """
        Check if this script is the same as another script. Will check the names of both scripts to see if they match.

        Arguments
        - other: The other script to check against this script

        Returns
        - True if the scripts are equal, False if otherwise
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.manager.listener.BukkitListenerManager

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit import PySpigot
from dev.magicmq.pyspigot.bukkit.manager.listener import *
from dev.magicmq.pyspigot.manager.listener import ListenerManager
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.util import ScriptUtils
from java.lang.reflect import InvocationTargetException
from java.lang.reflect import Method
from org.bukkit import Bukkit
from org.bukkit.event import Event
from org.bukkit.event import EventPriority
from org.bukkit.event import HandlerList
from org.python.core import PyFunction
from typing import Any, Callable, Iterable, Tuple


class BukkitListenerManager(ListenerManager):
    """
    The Bukkit-specific implementation of the listener manager.
    """

    def registerListener(self, function: Callable, eventClass: type["Event"]) -> "BukkitScriptEventListener":
        ...


    def registerListener(self, function: Callable, eventClass: type["Event"], priority: "EventPriority") -> "BukkitScriptEventListener":
        ...


    def registerListener(self, function: Callable, eventClass: type["Event"], ignoreCancelled: bool) -> "BukkitScriptEventListener":
        ...


    def registerListener(self, function: Callable, eventClass: type["Event"], priority: "EventPriority", ignoreCancelled: bool) -> "BukkitScriptEventListener":
        ...


    def unregisterListener(self, listener: "BukkitScriptEventListener") -> None:
        ...


    def unregisterListeners(self, script: "Script") -> None:
        ...


    def getListener(self, script: "Script", eventClass: type["Event"]) -> "BukkitScriptEventListener":
        ...


    @staticmethod
    def get() -> "BukkitListenerManager":
        """
        Get the singleton instance of this BukkitListenerManager.

        Returns
        - The instance
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.util.player.BukkitCommandSender

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit.util.player import *
from dev.magicmq.pyspigot.util.player import CommandSenderAdapter
from net.md_5.bungee.api.chat import BaseComponent
from org.bukkit.command import CommandSender
from org.bukkit.entity import Player
from typing import Any, Callable, Iterable, Tuple


class BukkitCommandSender(CommandSenderAdapter):
    """
    A wrapper for the Bukkit org.bukkit.command.CommandSender class.
    """

    def __init__(self, sender: "CommandSender"):
        """
        Arguments
        - sender: The Bukkit CommandSender
        """
        ...


    def hasPermission(self, permission: str) -> bool:
        ...


    def sendMessage(self, message: str) -> None:
        ...


    def sendMessage(self, message: list["BaseComponent"]) -> None:
        ...


    def isPlayer(self) -> bool:
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.database.DatabaseManager

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.mongodb import ConnectionString
from com.mongodb import MongoClientSettings
from com.zaxxer.hikari import HikariConfig
from dev.magicmq.pyspigot.manager.database import *
from dev.magicmq.pyspigot.manager.database.mongo import MongoDatabase
from dev.magicmq.pyspigot.manager.database.sql import SqlDatabase
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.util import ScriptUtils
from typing import Any, Callable, Iterable, Tuple


class DatabaseManager:
    """
    Manager that allows connection to and interact with a variety of database types. Primarily used by scripts to interact with external databases, such as SQL and MongoDB.
    """

    def newHikariConfig(self) -> "HikariConfig":
        """
        Get a new com.zaxxer.hikari.HikariConfig for specifying configuration options.
        
        **Note:** This should be called from scripts only!

        Returns
        - A new HikariConfig
        """
        ...


    def connectSql(self, host: str, port: str, database: str, username: str, password: str) -> "SqlDatabase":
        """
        Open a new connection with an SQL database, using the default configuration options.
        
        **Note:** This should be called from scripts only!

        Arguments
        - host: The host URL or IP of the SQL database
        - port: The port of the SQL database
        - database: The name of the SQL database
        - username: The username of the SQL database
        - password: The password of the SQL database

        Returns
        - An SqlDatabase object representing an open connection to the database
        """
        ...


    def connectSql(self, host: str, port: str, database: str, username: str, password: str, hikariConfig: "HikariConfig") -> "SqlDatabase":
        """
        Open a new connection with an SQL database, using the specified configuration options.
        
        **Note:** This should be called from scripts only!

        Arguments
        - host: The host URL or IP of the SQL database
        - port: The port of the SQL database
        - database: The name of the SQL database
        - username: The username of the SQL database
        - password: The password of the SQL database
        - hikariConfig: A com.zaxxer.hikari.HikariConfig object representing the configuration options for the connection

        Returns
        - An SqlDatabase object representing an open connection to the database
        """
        ...


    def connectSql(self, uri: str) -> "SqlDatabase":
        """
        Open a new connection with an SQL database, using the provided connection URI.
        
        **Note:** This should be called from scripts only!

        Arguments
        - uri: The connection string to define the connection, including options

        Returns
        - An SqlDatabase object representing an open connection to the database
        """
        ...


    def connectSql(self, uri: str, hikariConfig: "HikariConfig") -> "SqlDatabase":
        """
        Open a new connection with an SQL database, using the provided connection URI and configuration options.
        
        **Note:** This should be called from scripts only!

        Arguments
        - uri: The connection string to define the connection, including options
        - hikariConfig: A com.zaxxer.hikari.HikariConfig object representing the configuration options for the connection

        Returns
        - An SqlDatabase object representing an open connection to the database
        """
        ...


    def connectSql(self, hikariConfig: "HikariConfig") -> "SqlDatabase":
        """
        Open a new connection with an SQL database, using the provided configuration.
        
        **Note:** This should be called from scripts only!

        Arguments
        - hikariConfig: The configuration for the connection

        Returns
        - An SqlDatabase object representing an open connection to the database
        """
        ...


    def newMongoClientSettings(self) -> "MongoClientSettings.Builder":
        """
        Get a new com.mongodb.MongoClientSettings.Builder for specifying client settings.
        
        **Note:** This should be called from scripts only!

        Returns
        - A new MongoClientSettings builder
        """
        ...


    def connectMongo(self, host: str, port: str, username: str, password: str) -> "MongoDatabase":
        """
        Open a new connection with a Mongo database, using the default client settings.
        
        **Note:** This should be called from scripts only!

        Arguments
        - host: The host URL or IP of the Mongo database
        - port: The port of the Mongo database
        - username: The username of the Mongo database
        - password: The password of the Mongo database

        Returns
        - An MongoDatabase object representing an open connection to the database
        """
        ...


    def connectMongo(self, host: str, port: str, username: str, password: str, clientSettings: "MongoClientSettings") -> "MongoDatabase":
        """
        Open a new connection with a Mongo database, using the provided client settings.
        
        **Note:** This should be called from scripts only!

        Arguments
        - host: The host URL or IP of the Mongo database
        - port: The port of the Mongo database
        - username: The username of the Mongo database
        - password: The password of the Mongo database
        - clientSettings: A com.mongodb.MongoClientSettings object representing the client settings for the connection

        Returns
        - An MongoDatabase object representing an open connection to the database
        """
        ...


    def connectMongo(self, uri: str) -> "MongoDatabase":
        """
        Open a new connection with a Mongo database, using the provided connection string URI.
        
        **Note:** This should be called from scripts only!

        Arguments
        - uri: The connection string to define the connection, including options

        Returns
        - An MongoDatabase object representing an open connection to the database
        """
        ...


    def connectMongo(self, uri: str, clientSettings: "MongoClientSettings") -> "MongoDatabase":
        """
        Open a new connection with a Mongo database, using the provided connection string URI and client settings.
        
        **Note:** This should be called from scripts only!

        Arguments
        - uri: The connection string to define the connection, including options
        - clientSettings: A com.mongodb.MongoClientSettings object representing the client settings for the connection

        Returns
        - An MongoDatabase object representing an open connection to the database
        """
        ...


    def connectMongo(self, clientSettings: "MongoClientSettings") -> "MongoDatabase":
        """
        Open a new connection with a Mongo database, using the provided client settings.
        
        **Note:** This should be called from scripts only!

        Arguments
        - clientSettings: The client settings for the connection

        Returns
        - An MongoDatabase object representing an open connection to the database
        """
        ...


    def disconnect(self, connection: "Database") -> bool:
        """
        Disconnect from the provided database connection. Should be called when no longer using the database connection.
        
        **Note:** This should be called from scripts only!

        Arguments
        - connection: The database connection to disconnect from

        Returns
        - True if the disconnection was successful, False if otherwise
        """
        ...


    def disconnectAll(self, script: "Script") -> bool:
        """
        Disconnect from all database connections belonging to a certain script.

        Arguments
        - script: The script whose database connections should be disconnected

        Returns
        - True if all disconnections were successful, False if one or more connections were not closed successfully or if the script had no database connections to close
        """
        ...


    def getConnections(self, script: "Script") -> list["Database"]:
        """
        Get all database connnections belonging to a script.

        Arguments
        - script: The script to get database connections from

        Returns
        - An immutable List of Database containing all database connections associated with the script. Will return null if there are no open database connections associated with the script
        """
        ...


    def getConnections(self, script: "Script", type: "DatabaseType") -> list["Database"]:
        """
        Get all database connnections belonging to a script of the given type.

        Arguments
        - script: The script to get database connections from
        - type: The type of database connection to filter by

        Returns
        - An immutable List of Database containing all database connections of the given type associated with the script. Will return null if there are no open database connections of the given type associated with the script
        """
        ...


    @staticmethod
    def get() -> "DatabaseManager":
        """
        Get the singleton instance of this DatabaseManager.

        Returns
        - The instance
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.redis.client.RedisPubSubClient

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.manager.redis import ScriptPubSubListener
from dev.magicmq.pyspigot.manager.redis.client import *
from dev.magicmq.pyspigot.manager.script import Script
from io.lettuce.core import ClientOptions
from io.lettuce.core import RedisFuture
from io.lettuce.core import RedisURI
from io.lettuce.core.pubsub import StatefulRedisPubSubConnection
from java.util import Iterator
from org.python.core import PyFunction
from typing import Any, Callable, Iterable, Tuple


class RedisPubSubClient(ScriptRedisClient):
    """
    Extension of the ScriptRedisClient that provides pub/sub messaging capabilities.

    See
    - io.lettuce.core.pubsub.StatefulRedisPubSubConnection
    """

    def __init__(self, script: "Script", redisURI: "RedisURI", clientOptions: "ClientOptions"):
        """
        Arguments
        - script: The script to which this RedisPubSubClient belongs
        - redisURI: The URI that specifies the connection details to the server
        - clientOptions: The io.lettuce.core.ClientOptions that should be used for the RedisClient
        """
        ...


    def open(self) -> None:
        """

        """
        ...


    def getConnection(self) -> "StatefulRedisPubSubConnection"[str, str]:
        """
        Get the underlying connection for this RedisPubSubClient.

        Returns
        - The connection associated with this RedisPubSubClient
        """
        ...


    def registerListener(self, function: Callable, channel: str) -> "ScriptPubSubListener":
        """
        Register a new synchronous listener.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when a message on the specified channel is received
        - channel: The channel to listen on

        Returns
        - A ScriptPubSubListener representing the listener that was registered

        See
        - RedisPubSubClient.registerSyncListener(PyFunction, String)
        """
        ...


    def registerSyncListener(self, function: Callable, channel: str) -> "ScriptPubSubListener":
        """
        Register a new synchronous listener.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when a message on the specified channel is received
        - channel: The channel to listen on

        Returns
        - A ScriptPubSubListener representing the listener that was registered
        """
        ...


    def registerAsyncListener(self, function: Callable, channel: str) -> "ScriptPubSubListener":
        """
        Register a new asynchronous listener.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when a message on the specified channel is received
        - channel: The channel to listen on

        Returns
        - A ScriptPubSubListener representing the listener that was registered
        """
        ...


    def unregisterListener(self, listener: "ScriptPubSubListener") -> None:
        """
        Unregister the specified listener.
        
        **Note:** This should be called from scripts only!

        Arguments
        - listener: The listener to unregister
        """
        ...


    def unregisterListeners(self, channel: str) -> None:
        """
        Unregister all listeners (both synchronous and asynchronous) on the given channel
        
        **Note:** This should be called from scripts only!

        Arguments
        - channel: The channel on which all listeners should be unregistered
        """
        ...


    def publish(self, channel: str, message: str) -> "Long":
        """
        Synchronously publish a message to the given channel
        
        **Note:** This should be called from scripts only!

        Arguments
        - channel: The channel on which the message should be published
        - message: The message to publish

        Returns
        - The number of clients that received the message
        """
        ...


    def publishSync(self, channel: str, message: str) -> "Long":
        """
        Synchronously publish a message to the given channel
        
        **Note:** This should be called from scripts only!

        Arguments
        - channel: The channel on which the message should be published
        - message: The message to publish

        Returns
        - The number of clients that received the message
        """
        ...


    def publishAsync(self, channel: str, message: str) -> "RedisFuture"["Long"]:
        """
        Asynchronously publish a message to the given channel
        
        **Note:** This should be called from scripts only!

        Arguments
        - channel: The channel on which the message should be published
        - message: The message to publish
        """
        ...


    def toString(self) -> str:
        """
        Prints a representation of this RedisPubSubClient in string format, including listeners

        Returns
        - A string representation of the RedisPubSubClient
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.manager.protocol.ProtocolManager

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.comphenix.protocol import PacketType
from com.comphenix.protocol import ProtocolLibrary
from com.comphenix.protocol.events import ListenerPriority
from com.comphenix.protocol.events import PacketContainer
from dev.magicmq.pyspigot.bukkit.manager.protocol import *
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.util import ScriptUtils
from org.bukkit import Location
from org.bukkit.entity import Entity
from org.bukkit.entity import Player
from org.python.core import PyFunction
from typing import Any, Callable, Iterable, Tuple


class ProtocolManager:
    """
    Manager to interface with ProtocolLib's ProtocolManager. Primarily used by scripts to register and unregister packet listeners on Bukkit/Minecraft.
    
    Do not call this manager if ProtocolLib is not loaded and enabled on the server! It will not work.

    See
    - com.comphenix.protocol.ProtocolManager
    """

    def getProtocolManager(self) -> "com.comphenix.protocol.ProtocolManager":
        """
        Get the current ProtocolLib ProtocolManager.

        Returns
        - The com.comphenix.protocol.ProtocolManager
        """
        ...


    def async(self) -> "AsyncProtocolManager":
        """
        Get the async protocol manager for working with asynchronous listeners.

        Returns
        - The AsyncProtocolManager
        """
        ...


    def registerPacketListener(self, function: Callable, type: "PacketType") -> "ScriptPacketListener":
        """
        Register a new packet listener with default priority.
        
        This method will automatically register a PacketReceivingListener or a PacketSendingListener, depending on if the PacketType is for the server or client, respectively.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the packet event occurs
        - type: The packet type to listen for

        Returns
        - A ScriptPacketListener representing the packet listener that was registered
        """
        ...


    def registerPacketListener(self, function: Callable, type: "PacketType", priority: "ListenerPriority") -> "ScriptPacketListener":
        """
        Register a new packet listener.
        
        This method will automatically register a PacketReceivingListener or a PacketSendingListener, depending on if the PacketType is for the server or client, respectively.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the packet event occurs
        - type: The packet type to listen for
        - priority: The priority of the packet listener relative to other packet listeners

        Returns
        - A ScriptPacketListener representing the packet listener that was registered
        """
        ...


    def unregisterPacketListener(self, listener: "ScriptPacketListener") -> None:
        """
        Unregister a packet listener.
        
        **Note:** This should be called from scripts only!

        Arguments
        - listener: The packet listener to unregister
        """
        ...


    def unregisterPacketListeners(self, script: "Script") -> None:
        """
        Unregister all normal packet listeners belonging to a script, excluding asynchronous packet listeners.
        
        Use AsyncProtocolManager.unregisterAsyncPacketListeners(Script) to unregister asynchronous packet listeners.

        Arguments
        - script: The script whose normal packet listeners should be unregistered
        """
        ...


    def getPacketListeners(self, script: "Script") -> list["ScriptPacketListener"]:
        """
        Get all normal packet listeners associated with a script, excluding asynchronous packet listeners.
        
        Use AsyncProtocolManager.getAsyncPacketListeners(Script) to get a script's asynchronous packet listeners.

        Arguments
        - script: The script to get normal packet listeners from

        Returns
        - A List of ScriptPacketListener containing all normal packet listeners associated with this script. Will return null if there are no normal packet listeners associated with the script
        """
        ...


    def getPacketListener(self, script: "Script", packetType: "PacketType") -> "ScriptPacketListener":
        """
        Get the normal packet listener for a particular packet type associated with a script.
        
        Use AsyncProtocolManager.getAsyncPacketListener(Script, PacketType) to get a script's asynchronous packet listener of a specific packet type.

        Arguments
        - script: The script
        - packetType: The packet type

        Returns
        - The ScriptPacketListener associated with the script and packet type, null if there is none
        """
        ...


    def createPacket(self, type: "PacketType") -> "PacketContainer":
        """
        Create a new packet with the given type. This method will assign sensible default values to all fields within the packet where a non-null value is required.
        
        This method is the preferred way to create a packet that will later be sent or broadcasted.
        
        **Note:** This should be called from scripts only!

        Arguments
        - type: The type of packet to create

        Returns
        - A com.comphenix.protocol.events.PacketContainer representing the packet that was created.
        """
        ...


    def sendServerPacket(self, player: "Player", packet: "PacketContainer") -> None:
        """
        Send a packet to a player.
        
        **Note:** This should be called from scripts only!

        Arguments
        - player: The player to send the packet to
        - packet: The packet to send
        """
        ...


    def broadcastServerPacket(self, packet: "PacketContainer") -> None:
        """
        Broadcast a packet to the entire server. The packet will be sent to all online players.
        
        **Note:** This should be called from scripts only!

        Arguments
        - packet: The packet to broadcast
        """
        ...


    def broadcastServerPacket(self, packet: "PacketContainer", entity: "Entity") -> None:
        """
        Broadcast a packet to players receiving information about a particular entity. Will also broadcast the packet to the entity, if the entity is a tracker.
        
        **Note:** This should be called from scripts only!

        Arguments
        - packet: The packet to broadcast
        - entity: The entity whose trackers will be informed

        See
        - ProtocolManager.broadcastServerPacket(PacketContainer, Entity, boolean)
        """
        ...


    def broadcastServerPacket(self, packet: "PacketContainer", entity: "Entity", includeTracker: bool) -> None:
        """
        Broadcast a packet to players receiving information about a particular entity.
        
        Usually, this would be every player in the same world within an observable distance. If the entity is a player, it will be included only if `includeTracker` is set to `True`.
        
        **Note:** This should be called from scripts only!

        Arguments
        - packet: The packet to broadcast
        - entity: The entity whose trackers will be informed
        - includeTracker: Whether to also transmit the packet to the entity, if it is a tracker
        """
        ...


    def broadcastServerPacket(self, packet: "PacketContainer", origin: "Location", maxObserverDistance: int) -> None:
        """
        Broadcast a packet to all players within a given max observer distance from an origin location (center point).
        
        **Note:** This should be called from scripts only!

        Arguments
        - packet: The packet to broadcast
        - origin: The origin location (center point) to consider when calculating distance to each observer
        - maxObserverDistance: The maximum distance from origin wherein packets will be broadcasted
        """
        ...


    def broadcastServerPacket(self, packet: "PacketContainer", targetPlayers: Iterable["Player"]) -> None:
        """
        Broadcast a packet to a specified list of players.
        
        **Note:** This should be called from scripts only!

        Arguments
        - packet: The packet to broadcast
        - targetPlayers: The list of players to which the packet should be broadcasted
        """
        ...


    @staticmethod
    def get() -> "ProtocolManager":
        """
        Get the singleton instance of this ProtocolManager.

        Returns
        - The instance
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.PyCore

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot import *
from dev.magicmq.pyspigot.config import PluginConfig
from dev.magicmq.pyspigot.config import ScriptOptionsConfig
from dev.magicmq.pyspigot.manager.database import DatabaseManager
from dev.magicmq.pyspigot.manager.libraries import LibraryManager
from dev.magicmq.pyspigot.manager.redis import RedisManager
from dev.magicmq.pyspigot.manager.script import GlobalVariables
from dev.magicmq.pyspigot.manager.script import ScriptManager
from dev.magicmq.pyspigot.util import StringUtils
from java.io import File
from java.io import FileOutputStream
from java.io import IOException
from java.io import InputStream
from java.io import OutputStream
from java.net import URL
from java.nio.file import Path
from java.util import Scanner
from typing import Any, Callable, Iterable, Tuple


class PyCore:
    """
    Core class of PySpigot for all platform-specific implementations. Platform-specific code is implemented via the PlatformAdapter.

    See
    - PlatformAdapter
    """

    @staticmethod
    def newInstance(adapter: "PlatformAdapter") -> None:
        """
        Initialize the PyCore instance.
        
        Called from the `onEnable` method of the platform-specific plugin class (PySpigot for Bukkit, for example).

        Arguments
        - adapter: The platform-specific adapter.
        """
        ...


    def init(self) -> None:
        """
        Initialize the plugin.
        """
        ...


    def shutdown(self) -> None:
        """
        Shutdown the plugin.
        """
        ...


    def reloadConfigs(self) -> None:
        """
        Reload the plugin config and the script options config.
        """
        ...


    def getLogger(self) -> "Logger":
        """
        Get the logger for PySpigot.

        Returns
        - The logger
        """
        ...


    def getDataFolder(self) -> "File":
        """
        Get the data folder for PySpigot.

        Returns
        - The data folder
        """
        ...


    def getDataFolderPath(self) -> "Path":
        """
        Get the path of the data folder for PySpigot.

        Returns
        - A path representing the data folder
        """
        ...


    def getPluginClassLoader(self) -> "ClassLoader":
        """
        Get the ClassLoader for PySpigot.

        Returns
        - The ClassLoader
        """
        ...


    def getVersion(self) -> str:
        """
        Get the version of the plugin.

        Returns
        - The version
        """
        ...


    def getPluginIdentifier(self) -> str:
        """
        Get the identifier of the plugin.

        Returns
        - The plugin identifier
        """
        ...


    def isPaper(self) -> bool:
        """
        Get if the server is running paper.

        Returns
        - True if the server is running paper, False if otherwise
        """
        ...


    def getConfig(self) -> "PluginConfig":
        """
        Get the plugin configuration for PySpigot.

        Returns
        - The PySpigot plugin config
        """
        ...


    def getSpigotVersion(self) -> str:
        """
        Get the latest available plugin version on Spigot.

        Returns
        - The latest available version on Spigot
        """
        ...


    def fetchSpigotVersion(self) -> None:
        """
        Fetch the latest available plugin version from SpigotMC.
        """
        ...


    def compareVersions(self) -> None:
        """
        Compare the current loaded plugin version with the cached latest SpigotMC plugin version, and log a message to console if the current version is detected as outdated.
        """
        ...


    def saveResource(self, resourcePath: str, replace: bool) -> None:
        """
        Save a resource from the plugin JAR file to the plugin data folder.

        Arguments
        - resourcePath: The path of the resource to save
        - replace: True if the file should be replaced (if it already exists in the data folder), False if it should not
        """
        ...


    @staticmethod
    def get() -> "PyCore":
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.manager.config.BukkitScriptConfig

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit.manager.config import *
from dev.magicmq.pyspigot.manager.config import ScriptConfig
from java.io import File
from java.io import IOException
from java.nio.file import Path
from java.nio.file import Paths
from org.bukkit.configuration import InvalidConfigurationException
from org.bukkit.configuration.file import YamlConfiguration
from typing import Any, Callable, Iterable, Tuple


class BukkitScriptConfig(YamlConfiguration, ScriptConfig):
    """
    A class representing a script configuration file, for the Bukkit implementation.

    See
    - org.bukkit.configuration.file.YamlConfiguration
    """

    def __init__(self, configFile: "File", defaults: str):
        """
        Arguments
        - configFile: The configuration file
        - defaults: A YAML-formatted string containing the desired default values for the configuration
        """
        ...


    def getConfigFile(self) -> "File":
        ...


    def getConfigPath(self) -> "Path":
        ...


    def load(self) -> None:
        ...


    def reload(self) -> None:
        ...


    def save(self) -> None:
        ...


    def setIfNotExists(self, path: str, value: "Object") -> bool:
        """
        Sets the specified path to the given value only if the path is not already set in the config file. Any specified default values are ignored when checking if the path is set.

        Arguments
        - path: Path of the object to set
        - value: Value to set the path to

        Returns
        - True if the path was set to the value (in other words the path was not previously set), False if the path was not set to the value (in other words the path was already previously set)

        See
        - org.bukkit.configuration.ConfigurationSection.set(String, Object)
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.redis.ClientType

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.manager.redis import *
from dev.magicmq.pyspigot.manager.redis.client import RedisCommandClient
from dev.magicmq.pyspigot.manager.redis.client import RedisPubSubClient
from dev.magicmq.pyspigot.manager.redis.client import ScriptRedisClient
from enum import Enum
from typing import Any, Callable, Iterable, Tuple


class ClientType(Enum):
    """
    Utility enum to represent different types of redis clients available for scripts to use.
    """

    BASIC = (ScriptRedisClient)
    """
    A basic client type, used for initiating a standard RedisClient for further custom usage.
    """
    COMMAND = (RedisCommandClient)
    """
    A command client type, used for executing redis commands.
    """
    PUB_SUB = (RedisPubSubClient)
    """
    A pub/sub client type, used for publishing and subscribing to redis messaging.
    """


    def getClientClass(self) -> type["ScriptRedisClient"]:
        """
        Get the class that pertains to the client type. Will be a subclass of ScriptRedisClient

        Returns
        - The class associated with the client type
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.redis.ScriptPubSubListener

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.manager.redis import *
from io.lettuce.core.pubsub import RedisPubSubListener
from org.python.core import Py
from org.python.core import PyFunction
from org.python.core import PyObject
from typing import Any, Callable, Iterable, Tuple


class ScriptPubSubListener(RedisPubSubListener):
    """
    A wrapper class that wraps the RedisPubSubListener from lettuce for use by scripts.

    See
    - io.lettuce.core.pubsub.RedisPubSubListener
    """

    def __init__(self, function: Callable, channel: str):
        """
        Arguments
        - function: The function that should be called when a message is received on the given channel
        - channel: The channel to listen on
        """
        ...


    def message(self, channel: str, message: str) -> None:
        """
        Called internally when a message is received on the given channel.

        Arguments
        - channel: The channel on which the message was received
        - message: The message that was received
        """
        ...


    def message(self, s: str, k1: str, s2: str) -> None:
        """
        Implemented from RedisPubSubListener, but unused.
        """
        ...


    def subscribed(self, s: str, l: int) -> None:
        """
        Implemented from RedisPubSubListener, but unused.
        """
        ...


    def psubscribed(self, s: str, l: int) -> None:
        """
        Implemented from RedisPubSubListener, but unused.
        """
        ...


    def unsubscribed(self, s: str, l: int) -> None:
        """
        Implemented from RedisPubSubListener, but unused.
        """
        ...


    def punsubscribed(self, s: str, l: int) -> None:
        """
        Implemented from RedisPubSubListener, but unused.
        """
        ...


    def getChannel(self) -> str:
        """
        Implemented from RedisPubSubListener, but unused.
        """
        ...


    def toString(self) -> str:
        """
        Prints a representation of this ScriptPubSubListener in string format, including the channel being listened to.

        Returns
        - A string representation of the ScriptPubSubListener
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.event.ScriptEvent

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit.event import *
from dev.magicmq.pyspigot.manager.script import Script
from org.bukkit.event import Event
from org.bukkit.event import HandlerList
from typing import Any, Callable, Iterable, Tuple


class ScriptEvent(Event):
    """
    Script event superclass. All script events inherit from this class.
    """

    def __init__(self, script: "Script", async: bool):
        """
        Arguments
        - script: The script associated with this event
        - async: Whether the event is asynchronous
        """
        ...


    def getScript(self) -> "Script":
        """
        Get the script associated with this event.

        Returns
        - The script associated with this event
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.database.Database

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.manager.database import *
from dev.magicmq.pyspigot.manager.script import Script
from typing import Any, Callable, Iterable, Tuple


class Database:
    """
    Represents a database to which a script is connected and can read/write.
    """

    def __init__(self, script: "Script"):
        """
        Arguments
        - script: The script associated with this database connection
        """
        ...


    def open(self) -> bool:
        """
        Opens a connection to the database.

        Returns
        - True if opening the connection to the database was successful, False if otherwise
        """
        ...


    def close(self) -> bool:
        """
        Closes a connection to the database.

        Returns
        - True if closing the connection to the database was successful, False if otherwise
        """
        ...


    def getScript(self) -> "Script":
        """
        Get the script associated with this database connection.

        Returns
        - The script associated with this database connection
        """
        ...


    def getDatabaseId(self) -> int:
        """
        Get the ID of this database connection.

        Returns
        - The ID
        """
        ...


    def toString(self) -> str:
        """
        Prints a representation of this Database in string format.

        Returns
        - A string representation of this Database
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.script.GlobalVariables

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.manager.script import *
from typing import Any, Callable, Iterable, Tuple


class GlobalVariables:
    """
    A wrapper class for a HashMap that contains global variables that can be shared across scripts.

    See
    - java.util.HashMap
    """

    def set(self, key: str, value: "Object") -> "Object":
        """
        Set a global variable. Will override an existing global variable with the same name.

        Arguments
        - key: The name of the variable to set
        - value: The value of the variable

        Returns
        - The value that was previously associated with the given key, or null if there was none
        """
        ...


    def set(self, key: str, value: "Object", override: bool) -> "Object":
        """
        Set a global variable, with the option to override an existing global variable with the same name.

        Arguments
        - key: The name of the variable to set
        - value: The value of the variable
        - override: Whether an existing value should be overridden

        Returns
        - If override is True, will return the value that was previously associated with the key, or null if there was none. If override is False, will return the existing value, or null if there was none
        """
        ...


    def remove(self, key: str) -> "Object":
        """
        Remove/delete a global variable.

        Arguments
        - key: The name of the variable to remove

        Returns
        - The value of the variable that was removed, or null if there was none
        """
        ...


    def get(self, key: str) -> "Object":
        """
        Get a global variable

        Arguments
        - key: The name of the variable to get

        Returns
        - The variable, or null if there is no variable associated with the given key
        """
        ...


    def getKeys(self) -> set[str]:
        """
        Get a set of all global variable names.

        Returns
        - An immutable java.util.Set containing the names of all global variables. Will return an empty set if there are no global variables
        """
        ...


    def getValues(self) -> Iterable["Object"]:
        """
        Get a set of all global variable values.

        Returns
        - An immutable java.util.Collection containing the values of all global variables. Will return an empty collection if there are no global variables
        """
        ...


    def getHashMap(self) -> dict[str, "Object"]:
        """
        Get the underlying java.util.HashMap wherein global variables are cached.

        Returns
        - The underlying HashMap, which is mutable
        """
        ...


    def contains(self, key: str) -> bool:
        """
        Check if a global variable exists with the given name.

        Arguments
        - key: The name to check

        Returns
        - True if there is a global variable with the given name, False if there is not
        """
        ...


    def containsValue(self, value: "Object") -> bool:
        """
        Check if a global variable exists with the given value.

        Arguments
        - value: The value to check

        Returns
        - True if there is a global variable with the given value, False if there is not
        """
        ...


    def purge(self) -> None:
        """
        Clear all global variables.
        """
        ...


    @staticmethod
    def get() -> "GlobalVariables":
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.command.ScriptCommand

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.manager.command import *
from dev.magicmq.pyspigot.manager.script import Script
from typing import Any, Callable, Iterable, Tuple


class ScriptCommand:
    """
    A command registered by a script. Meant to be implemented by platform-specific classes that also implement or extend an API's command executor class/interface.
    """

    def getScript(self) -> "Script":
        """
        Get the script associated with this command.

        Returns
        - The script associated with this command
        """
        ...


    def getName(self) -> str:
        """
        Get the name of this command.

        Returns
        - The name of this command
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.util.StringUtils

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.util import *
from java.time import Duration
from typing import Any, Callable, Iterable, Tuple


class StringUtils:
    """
    A utility class for various methods/classes related to Strings.
    """

    @staticmethod
    def replaceLastOccurrence(string: str, toReplace: str, replaceWith: str) -> str:
        ...


    @staticmethod
    def formatDuration(duration: "Duration") -> str:
        ...


    class Version(Comparable):

        def __init__(self, version: str):
            ...


        def getVersion(self) -> str:
            ...


        def compareTo(self, that: "StringUtils.Version") -> int:
            ...



from .UnloadCommand import UnloadCommand
from .ReloadConfigCommand import ReloadConfigCommand
from .ReloadCommand import ReloadCommand
from .ReloadAllCommand import ReloadAllCommand
from .LoadLibraryCommand import LoadLibraryCommand
from .LoadCommand import LoadCommand
from .ListScriptsCommand import ListScriptsCommand
from .InfoCommand import InfoCommand
from .HelpCommand import HelpCommand


"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.script.ScriptManager

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot import PyCore
from dev.magicmq.pyspigot.exception import InvalidConfigurationException
from dev.magicmq.pyspigot.manager.command import CommandManager
from dev.magicmq.pyspigot.manager.database import DatabaseManager
from dev.magicmq.pyspigot.manager.libraries import LibraryManager
from dev.magicmq.pyspigot.manager.listener import ListenerManager
from dev.magicmq.pyspigot.manager.redis import RedisManager
from dev.magicmq.pyspigot.manager.script import *
from dev.magicmq.pyspigot.manager.task import TaskManager
from java.io import FileInputStream
from java.io import IOException
from java.nio.file import Files
from java.nio.file import Path
from java.util import Collections
from java.util import SortedSet
from java.util.stream import Stream
from org.python.core import Py
from org.python.core import PyBaseCode
from org.python.core import PyException
from org.python.core import PyFunction
from org.python.core import PyIndentationError
from org.python.core import PyObject
from org.python.core import PySyntaxError
from org.python.core import PySystemState
from typing import Any, Callable, Iterable, Tuple


class ScriptManager:
    """
    Master manager class for PySpigot. Contains all logic to load, unload, and reload scripts.
    
    Internally, utilizes Jython's org.python.util.PythonInterpreter to run scripts.

    See
    - Script
    """

    def scheduleStartScriptTask(self) -> None:
        """
        Schedules and starts the start script task for the script load delay via a platform-specific implementation.
        """
        ...


    def cancelStartScriptTask(self) -> None:
        """
        Cancels the start script task via a platform-specific implementation.
        """
        ...


    def isPluginDependencyMissing(self, dependency: str) -> bool:
        """
        Checks if a script plugin dependency is missing via a platform-specific implementation.

        Arguments
        - dependency: The name of the dependency to check

        Returns
        - True if the dependency is missing, False if it is present
        """
        ...


    def callScriptExceptionEvent(self, script: "Script", exception: "PyException") -> bool:
        """
        Calls a ScriptExceptionEvent via a platform-specific implementation.

        Arguments
        - script: The script that threw the exception
        - exception: The exception that was thrown

        Returns
        - True if the exception should be reported, False if otherwise
        """
        ...


    def callScriptLoadEvent(self, script: "Script") -> None:
        """
        Calls a ScriptLoadEvent via a platform-specific implementation.

        Arguments
        - script: The script that was loaded
        """
        ...


    def callScriptUnloadEvent(self, script: "Script", error: bool) -> None:
        """
        Calls a ScriptUnloadEvent via a platform-specific implementation.

        Arguments
        - script: The script that was unloaded
        - error: True if the unload was due to an error, False if it was not
        """
        ...


    def newScriptOptions(self) -> "ScriptOptions":
        """
        Initialize a new ScriptOptions with the default values.
        
        This is done in a platform-specific implementation, as initializing script options for Bukkit initializes permissions

        Returns
        - The new ScriptOptions
        """
        ...


    def newScriptOptions(self, scriptName: str) -> "ScriptOptions":
        """
        Initialize a new ScriptOptions using the appropriate values in the script_options.yml file, using the script name to search for the values.
        
        This is done in a platform-specific implementation, as initializing script options for Bukkit initializes permissions

        Arguments
        - scriptName: The name of the script whose script options should be initialized

        Returns
        - The new ScriptOptions

        Raises
        - InvalidConfigurationException: If
        """
        ...


    def newScript(self, path: "Path", name: str, options: "ScriptOptions") -> "Script":
        """
        Initialize a new Script via a platform-specific implementation.

        Arguments
        - path: The path that corresponds to the file where the script lives
        - name: The name of this script. Should contain its extension (.py)
        - options: The ScriptOptions for this script

        Returns
        - The new script
        """
        ...


    def initScriptPermissions(self, script: "Script") -> None:
        """
        Initialize script permissions via a platform-specific implementation.

        Arguments
        - script: The script whose permissions should be initialized
        """
        ...


    def removeScriptPermissions(self, script: "Script") -> None:
        """
        Remove script permissions from the server via a platform-specific implementation.

        Arguments
        - script: The script whose permissions should be removed
        """
        ...


    def unregisterFromPlatformManagers(self, script: "Script") -> None:
        """
        Unregisters the script from any platform-specific managers.

        Arguments
        - script: The script to unregister
        """
        ...


    def initJython(self) -> None:
        """
        Initialize Jython. Will only initialize once; subsequent calls to this method have no effect.
        """
        ...


    def shutdown(self) -> None:
        """
        Called on plugin unload or server shutdown. Gracefully stops and unloads all loaded and running scripts.
        """
        ...


    def loadScripts(self) -> None:
        """
        Loads and runs all scripts contained within the scripts folder. Called on plugin load (I.E. during server start). Loads in the appropriate load order (see Script.compareTo(Script)).
        """
        ...


    def getScriptOptions(self, name: str) -> "ScriptOptions":
        """
        Get the ScriptOptions for a particular script from the script file name.

        Arguments
        - name: The name of the script to fetch options for. Name should contain the file extension (.py)

        Returns
        - A ScriptOptions object representing the options beloinging to the script, or null if no script file was found that matched the given name
        """
        ...


    def getScriptOptions(self, path: "Path") -> "ScriptOptions":
        """
        Get the ScriptOptions for a particular script from the path pointing to the script file.

        Arguments
        - path: The path pointing to the script file to get script options for

        Returns
        - A ScriptOptions object representing the options beloinging to the script, or null if no script file was found that matched the given path
        """
        ...


    def loadScript(self, name: str) -> "RunResult":
        """
        Load a script with the given name.

        Arguments
        - name: The file name of the script to load. Name should contain the file extension (.py)

        Returns
        - A RunResult describing the outcome of the load operation

        Raises
        - IOException: If there was an IOException related to loading the script file
        """
        ...


    def loadScript(self, path: "Path") -> "RunResult":
        """
        Load a script with the given path.

        Arguments
        - path: The absolute path pointing to the script file to load.

        Returns
        - A RunResult describing the outcome of the load operation

        Raises
        - IOException: If there was an IOException related to loading the script file
        """
        ...


    def loadScript(self, script: "Script") -> "RunResult":
        """
        Load the given script.

        Arguments
        - script: The script that should be loaded

        Returns
        - A RunResult describing the outcome of the load operation

        Raises
        - IOException: If there was an IOException related to loading the script file
        """
        ...


    def unloadScripts(self) -> None:
        """
        Unload all currently loaded scripts. Unloads in the reverse order that they were loaded (I.E. the opposite of the load order).
        """
        ...


    def unloadScript(self, name: str) -> bool:
        """
        Unload a script with the given name.

        Arguments
        - name: The name of the script to unload. Name should contain the script file extension (.py)

        Returns
        - True if the script was successfully unloaded, False if otherwise
        """
        ...


    def unloadScript(self, script: "Script", error: bool) -> bool:
        """
        Unload a given script.

        Arguments
        - script: The script to unload
        - error: If the script unload was due to an error, pass True. Otherwise, pass False. This value will be passed on to a ScriptUnloadEvent

        Returns
        - True if the script was successfully unloaded, False if otherwise
        """
        ...


    def handleScriptException(self, script: "Script", exception: "PyException", message: str) -> None:
        """
        Handles script errors/exceptions, particularly for script logging purposes. Will also attempt to get the traceback of the org.python.core.PyException that was thrown and print it (if it exists).
        
        **Note:** This method will always run synchronously. If it is called from an asynchronous context, it will run inside a synchronous task.

        Arguments
        - script: The script that threw the error
        - exception: The PyException that was thrown
        - message: The message associated with the exception
        """
        ...


    def isScriptRunning(self, name: str) -> bool:
        """
        Check if a script with the given name is currently loaded.

        Arguments
        - name: The name of the script to check. Name should contain the script file extension (.py)

        Returns
        - True if the script is running, False if otherwise
        """
        ...


    def getScriptPath(self, name: str) -> "Path":
        """
        Attempts to resolve the absolute path for a script in the scripts folder based on the file name by searching through the scripts folder. Subfolders are also searched. If there are multiple matching files in different subfolders, the first match will be returned.

        Arguments
        - name: The name of the script file to search for

        Returns
        - The absolute path of the matching file, or null if no matching file was found
        """
        ...


    def getScript(self, name: str) -> "Script":
        """
        Get a Script object for a loaded and running script

        Arguments
        - name: The name of the script to get. Name should contain the script file extension (.py)

        Returns
        - The Script object for the script, null if no script is loaded and running with the given name
        """
        ...


    def getLoadedScripts(self) -> set["Script"]:
        """
        Get all loaded scripts.

        Returns
        - An immutable set containing all loaded and running scripts
        """
        ...


    def getLoadedScriptNames(self) -> set[str]:
        """
        Get the names of all loaded scripts.

        Returns
        - An immutable list containing the names of all loaded and running scripts
        """
        ...


    def getAllScriptPaths(self) -> "SortedSet"["Path"]:
        """
        Get a set of absolute paths corresponding to all script files in the scripts folder (including in subfolders).

        Returns
        - An immutable java.util.SortedSet of Paths representing the absolute paths of all script files
        """
        ...


    def getAllScriptNames(self) -> "SortedSet"[str]:
        """
        Get a set of file names corresponding to all script files in the scripts folder (including in subfolders). This only returns the names of the files, and does not include the subfolder.

        Returns
        - An immutable java.util.SortedSet of Strings representing the names of all script files (including in subfolders)
        """
        ...


    def getScriptInfo(self) -> "ScriptInfo":
        """
        Get the platform-specific script info object (for printing script info via the /pyspigot info command).

        Returns
        - The ScriptInfo
        """
        ...


    @staticmethod
    def get() -> "ScriptManager":
        """
        Get the singleton instance of this ScriptManager.

        Returns
        - The instance
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.util.ScriptUtils

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot import PyCore
from dev.magicmq.pyspigot.manager.libraries import LibraryManager
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.manager.script import ScriptManager
from dev.magicmq.pyspigot.util import *
from java.util import Optional
from org.python.core import PyString
from org.python.core import PySystemState
from typing import Any, Callable, Iterable, Tuple


class ScriptUtils:
    """
    A collection of utility methods related to scripts.
    """

    @staticmethod
    def getScriptFromCallStack() -> "Script":
        """
        Attempts to get the script involved in a Java method call by analyzing the call stack.

        Returns
        - The script associated with the method call, or null if no script was found in the call stack
        """
        ...


    @staticmethod
    def initPySystemState() -> "PySystemState":
        """
        Initializes a new PySystemState for a new org.python.util.PythonInterpreter when a script is loaded. This method will also initialize Jython if it hasn't been initialized previously.
        
        This method will also do the following with the new PySystemState: set its class loader to the class loader provided by the LibraryManager, and add "./plugins/PySpigot/python-libs/" and "./plugins/PySpigot/scripts/" to the path.

        Returns
        - The PySystemState that was created
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.manager.command.BukkitScriptCommand

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit import PySpigot
from dev.magicmq.pyspigot.bukkit.manager.command import *
from dev.magicmq.pyspigot.bukkit.util import CommandAliasHelpTopic
from dev.magicmq.pyspigot.manager.command import ScriptCommand
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.manager.script import ScriptManager
from java.lang.reflect import Constructor
from java.lang.reflect import Field
from java.lang.reflect import InvocationTargetException
from java.util import Collections
from org.bukkit import Bukkit
from org.bukkit import ChatColor
from org.bukkit.command import Command
from org.bukkit.command import CommandSender
from org.bukkit.command import PluginCommand
from org.bukkit.command import TabExecutor
from org.bukkit.help import GenericCommandHelpTopic
from org.bukkit.help import HelpMap
from org.bukkit.help import HelpTopic
from org.bukkit.help import HelpTopicComparator
from org.bukkit.help import IndexHelpTopic
from org.bukkit.plugin import Plugin
from org.python.core import Py
from org.python.core import PyBoolean
from org.python.core import PyException
from org.python.core import PyFunction
from org.python.core import PyList
from org.python.core import PyObject
from typing import Any, Callable, Iterable, Tuple


class BukkitScriptCommand(TabExecutor, ScriptCommand):
    """
    Represents a registered Bukkit command belonging to a script.

    See
    - org.bukkit.command.defaults.BukkitCommand
    """

    def __init__(self, script: "Script", commandFunction: Callable, tabFunction: Callable, name: str, description: str, usage: str, aliases: list[str], permission: str):
        """
        Arguments
        - script: The script to which this command belongs
        - commandFunction: The command function that should be called when the command is executed
        - tabFunction: The tab function that should be called for tab completion of the command. Can be null
        - name: The name of the command to register
        - description: The description of the command. Use an empty string for no description
        - usage: The usage message for the command
        - aliases: A List of String containing all the aliases for this command. Use an empty list for no aliases
        - permission: The required permission node to use this command. Can be null
        """
        ...


    def onCommand(self, sender: "CommandSender", cmd: "Command", label: str, args: list[str]) -> bool:
        ...


    def onTabComplete(self, sender: "CommandSender", cmd: "Command", alias: str, args: list[str]) -> list[str]:
        ...


    def getScript(self) -> "Script":
        """
        Get the script associated with this command.

        Returns
        - The script associated with this command
        """
        ...


    def getName(self) -> str:
        """
        Get the name of this command.

        Returns
        - The name of this command
        """
        ...


    def getBukkitCommand(self) -> "PluginCommand":
        """
        Get the org.bukkit.command.PluginCommand that underlies this ScriptCommand

        Returns
        - The underlying PluginCommand
        """
        ...


    def toString(self) -> str:
        """
        Prints a representation of this ScriptCommand in string format, including all variables that pertain to the command (such as name, label, description, etc.)

        Returns
        - A string representation of the ScriptCommand
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.manager.protocol.ScriptPacketListener

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.comphenix.protocol import PacketType
from com.comphenix.protocol.events import ListenerPriority
from com.comphenix.protocol.events import PacketAdapter
from com.comphenix.protocol.events import PacketEvent
from dev.magicmq.pyspigot.bukkit import PySpigot
from dev.magicmq.pyspigot.bukkit.manager.protocol import *
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.manager.script import ScriptManager
from org.python.core import Py
from org.python.core import PyException
from org.python.core import PyFunction
from org.python.core import PyObject
from typing import Any, Callable, Iterable, Tuple


class ScriptPacketListener(PacketAdapter):
    """
    An abstract class designed to represent a basic script packet listener.

    See
    - com.comphenix.protocol.events.PacketAdapter
    """

    def __init__(self, script: "Script", function: Callable, packetType: "PacketType", listenerPriority: "ListenerPriority", listenerType: "ListenerType"):
        """
        Arguments
        - script: The script associated with this packet listener
        - function: The function to be called when the packet event occurs
        - packetType: The packet type to listen for
        - listenerPriority: The com.comphenix.protocol.events.ListenerPriority of this listener
        - listenerType: The ListenerType of this listener
        """
        ...


    def getScript(self) -> "Script":
        """
        Get the script associated with this listener.

        Returns
        - The script associated with this listener
        """
        ...


    def getFunction(self) -> Callable:
        """
        Get the function that should be called when the packet event occurs.

        Returns
        - The function that should be called
        """
        ...


    def getPacketType(self) -> "PacketType":
        """
        Get the packet type being listener for.

        Returns
        - The packet type beign listened for
        """
        ...


    def getListenerType(self) -> "ListenerType":
        """
        The listener type of this listener.

        Returns
        - The ListenerType of this listener
        """
        ...


    def callToScript(self, event: "PacketEvent") -> None:
        """
        A helper method to call a script's packet listener function when the packet event occurs.

        Arguments
        - event: The event that occurred, will be passed to the script's function
        """
        ...


    def toString(self) -> str:
        """
        Prints a representation of this ScriptPacketListener in string format, including the packet type listened to by the listener

        Returns
        - A string representation of the ScriptPacketListener
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.manager.protocol.AsyncProtocolManager

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.comphenix.protocol import AsynchronousManager
from com.comphenix.protocol import PacketType
from com.comphenix.protocol import ProtocolLibrary
from com.comphenix.protocol.async import AsyncListenerHandler
from com.comphenix.protocol.events import ListenerPriority
from dev.magicmq.pyspigot.bukkit.manager.protocol import *
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.util import ScriptUtils
from org.python.core import PyFunction
from typing import Any, Callable, Iterable, Tuple


class AsyncProtocolManager:
    """
    Manager to interface with ProtocolLib's AsynchronousManager. Primarily used by scripts to register and unregister asynchronous packet listeners on Bukkit/Minecraft.

    See
    - com.comphenix.protocol.AsynchronousManager
    """

    def getAsynchronousManager(self) -> "AsynchronousManager":
        """
        Get the current ProtocolLib AsynchronousManager.

        Returns
        - The com.comphenix.protocol.AsynchronousManager
        """
        ...


    def registerAsyncPacketListener(self, function: Callable, type: "PacketType") -> "ScriptPacketListener":
        """
        Register a new asynchronous packet listener with default priority.
        
        This method will automatically register a PacketReceivingListener or a PacketSendingListener, depending on if the PacketType is for the server or client, respectively.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the packet event occurs
        - type: The packet type to listen for

        Returns
        - A ScriptPacketListener representing the asynchronous packet listener that was registered
        """
        ...


    def registerAsyncPacketListener(self, function: Callable, type: "PacketType", priority: "ListenerPriority") -> "ScriptPacketListener":
        """
        Register a new asynchronous packet listener.
        
        This method will automatically register a PacketReceivingListener or a PacketSendingListener, depending on if the PacketType is for the server or client, respectively.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the packet event occurs
        - type: The packet type to listen for
        - priority: The priority of the asynchronous packet listener relative to other packet listeners

        Returns
        - A ScriptPacketListener representing the asynchronous packet listener that was registered
        """
        ...


    def registerTimeoutPacketListener(self, function: Callable, type: "PacketType") -> "ScriptPacketListener":
        """
        Register a new asynchronous timeout packet listener with default priority.
        
        This method will automatically register a PacketReceivingListener or a PacketSendingListener, depending on if the PacketType is for the server or client, respectively.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the packet event occurs
        - type: The packet type to listen for

        Returns
        - A ScriptPacketListener representing the asynchronous timeout packet listener that was registered
        """
        ...


    def registerTimeoutPacketListener(self, function: Callable, type: "PacketType", priority: "ListenerPriority") -> "ScriptPacketListener":
        """
        Register a new asynchronous timeout packet listener.
        
        This method will automatically register a PacketReceivingListener or a PacketSendingListener, depending on if the PacketType is for the server or client, respectively.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the packet event occurs
        - type: The packet type to listen for
        - priority: The priority of the asynchronous timeout packet listener relative to other asynchronous timeout packet listeners

        Returns
        - A ScriptPacketListener representing the asynchronous timeout packet listener that was registered
        """
        ...


    def unregisterAsyncPacketListener(self, listener: "ScriptPacketListener") -> None:
        """
        Unregister an asynchronous packet listener.
        
        **Note:** This should be called from scripts only!

        Arguments
        - listener: The asynchronous packet listener to unregister
        """
        ...


    def unregisterAsyncPacketListeners(self, script: "Script") -> None:
        """
        Unregister all asynchronous packet listeners belonging to a script.

        Arguments
        - script: The script whose asynchronous packet listeners should be unregistered
        """
        ...


    def getAsyncPacketListeners(self, script: "Script") -> list["ScriptPacketListener"]:
        """
        Get all asynchronous packet listeners associated with a script

        Arguments
        - script: The script to get asynchronous packet listeners from

        Returns
        - A List of ScriptPacketListener containing all asynchronous packet listeners associated with this script. Will return null if there are no asynchronous packet listeners associated with the script
        """
        ...


    def getAsyncPacketListener(self, script: "Script", packetType: "PacketType") -> "ScriptPacketListener":
        """
        Get the asynchronous packet listener for a particular packet type associated with a script

        Arguments
        - script: The script
        - packetType: The packet type

        Returns
        - The ScriptPacketListener associated with the script and packet type, null if there is none
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.database.sql.SqlDatabase

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.zaxxer.hikari import HikariConfig
from com.zaxxer.hikari import HikariDataSource
from dev.magicmq.pyspigot.manager.database import Database
from dev.magicmq.pyspigot.manager.database.sql import *
from dev.magicmq.pyspigot.manager.script import Script
from typing import Any, Callable, Iterable, Tuple


class SqlDatabase(Database):
    """
    Represents an open connection to an SQL database.
    """

    def __init__(self, script: "Script", hikariConfig: "HikariConfig"):
        """
        Arguments
        - script: The script associated with this SQLDatabase
        - hikariConfig: The configuration options for the SQLDatabase connection
        """
        ...


    def open(self) -> bool:
        """

        """
        ...


    def close(self) -> bool:
        """

        """
        ...


    def getHikariDataSource(self) -> "HikariDataSource":
        """
        Get the underlying com.zaxxer.hikari.HikariDataSource associated with this SQLDatabase.

        Returns
        - The underlying HikariDataSource
        """
        ...


    def select(self, sql: str) -> dict[str, list["Object"]]:
        """
        Select from the SQL database.
        
        **Note:** This should be called from scripts only!

        Arguments
        - sql: The select statement

        Returns
        - A java.util.Map containing the data returned from the selection. Functionally identical to a python dict, where keys are column names and values are column data, with preserved order

        Raises
        - SQLException: If there was an exception when selecting from the database
        """
        ...


    def select(self, sql: str, values: list["Object"]) -> dict[str, list["Object"]]:
        """
        Select from the SQL database with the provided values that should be inserted into the select statement.
        
        **Note:** This should be called from scripts only!

        Arguments
        - sql: The select statement
        - values: The values that should be inserted into the select statement

        Returns
        - A java.util.Map containing the data returned from the selection. Functionally identical to a python dict, where keys are column names and values are column data, with preserved order

        Raises
        - SQLException: If there was an exception when selecting from the database
        """
        ...


    def update(self, sql: str) -> int:
        """
        Update the SQL database.
        
        **Note:** This should be called from scripts only!

        Arguments
        - sql: The update statement

        Returns
        - The number of rows that were affected by the update statement

        Raises
        - SQLException: If there was an exception when updating the database
        """
        ...


    def update(self, sql: str, values: list["Object"]) -> int:
        """
        Update the SQL database with the provided values that should be inserted into the update statement.
        
        **Note:** This should be called from scripts only!

        Arguments
        - sql: The update statement
        - values: The values that should be inserted into the update statement

        Returns
        - The number of rows that were affected by the update statement

        Raises
        - SQLException: If there was an exception when updating the database
        """
        ...


    def toString(self) -> str:
        """
        Prints a representation of this SqlDatabase in string format, including the ID, URI, and com.zaxxer.hikari.HikariDataSource

        Returns
        - A string representation of the SqlDatabase
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.config.ConfigManager

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot import PyCore
from dev.magicmq.pyspigot.manager.config import *
from java.io import IOException
from java.nio.file import Files
from java.nio.file import Path
from java.nio.file import Paths
from typing import Any, Callable, Iterable, Tuple


class ConfigManager:
    """
    Manager for scripts to interface with configuration files. Primarily used by scripts to load, write to, and save .yml files.
    """

    def loadConfig(self, filePath: str) -> "ScriptConfig":
        """
        Load a configuration file with the given path/name, relative to the `configs` folder. If the configuration file exists, it will load the existing file. If the configuration file does not exist, a new file will be created with the given path/name.
        
        **Note:** This should be called from scripts only!

        Arguments
        - filePath: The path of the configuration file to load, can be either the file name alone or a path (containing subfolders)

        Returns
        - A ScriptConfig representing the configuration file that was loaded

        Raises
        - IOException: If there was an IOException when attempting to load the configuration
        """
        ...


    def loadConfig(self, filePath: str, defaults: str) -> "ScriptConfig":
        """
        Load a configuration file with the given path/name, relative to the `configs` folder. If the configuration file exists, it will load the existing file. If the configuration file does not exist, a new file will be created with the given path/name.
        
        **Note:** This should be called from scripts only!

        Arguments
        - filePath: The path of the configuration file to load, can be either the file name alone or a path (containing subfolders)
        - defaults: A YAML-formatted string containing the desired default values for the configuration

        Returns
        - A ScriptConfig representing the configuration file that was loaded

        Raises
        - IOException: If there was an IOException when attempting to load the configuration
        """
        ...


    def doesConfigExist(self, filePath: str) -> bool:
        """
        Check if a configuration file exists with the given path/name, relative to the `configs` folder.
        
        **Note:** This should be called from scripts only!

        Arguments
        - filePath: The path of the configuration file to check, can be either the file name alone or a path (containing subfolders)

        Returns
        - True if the file exists, False if it does not
        """
        ...


    def createConfigIfNotExists(self, filePath: str) -> "Path":
        """
        Creates a config file with the given path if it did not previously exist.

        Arguments
        - filePath: The file path for the config file to create

        Returns
        - The Path to the config file

        Raises
        - IOException: If there was an IOException when attempting to create the config file
        """
        ...


    def deleteConfig(self, filePath: str) -> bool:
        """
        Delete a configuration file with the given path/name.
        
        **Note:** This should be called from scripts only!

        Arguments
        - filePath: The path of the configuration file to delete, relative to the `configs` folder. Can be either the file name alone or a path (containing subfolders)

        Returns
        - True if the file was deleted, False if the file could not be deleted because it does not exist

        Raises
        - IOException: If there was an IOException when attempting to delete the file
        """
        ...


    def getConfigFolder(self) -> "Path":
        """
        Get the path of the folder where script configuration files are stored.

        Returns
        - The path of the folder where script configuration files are stored
        """
        ...


    @staticmethod
    def get() -> "ConfigManager":
        """
        Get the singleton instance of this ConfigManager.

        Returns
        - The instance
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.script.RunResult

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.manager.script import *
from enum import Enum
from typing import Any, Callable, Iterable, Tuple


class RunResult(Enum):
    """
    An enum class representing various outcomes of running a script, including run failures.
    """

    SUCCESS = 0
    """
    Returned if the script loaded successfully.
    """
    FAIL_DISABLED = 1
    """
    Returned if the script was not loaded because it was disabled as per its script options in script_options.yml
    """
    FAIL_PLUGIN_DEPENDENCY = 2
    """
    Returned if the script was not loaded because it has one or more missing plugin dependencies.
    """
    FAIL_ERROR = 3
    """
    Returned if the script was loaded but failed during runtime due to an error.
    """
    FAIL_DUPLICATE = 4
    """
    Returned if a script is already loaded with the same name.
    """
    FAIL_SCRIPT_NOT_FOUND = 5
    """
    Returned if a script was not found with the given name.
    """



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.manager.task.BukkitTaskManager

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit import PySpigot
from dev.magicmq.pyspigot.bukkit.manager.task import *
from dev.magicmq.pyspigot.manager.task import RepeatingTask
from dev.magicmq.pyspigot.manager.task import SyncCallbackTask
from dev.magicmq.pyspigot.manager.task import Task
from dev.magicmq.pyspigot.manager.task import TaskManager
from org.bukkit import Bukkit
from typing import Any, Callable, Iterable, Tuple


class BukkitTaskManager(TaskManager):
    """
    The Bukkit-specific implementation of the task manager.
    """

    @staticmethod
    def get() -> "BukkitTaskManager":
        """
        Get the singleton instance of this BukkitTaskManager.

        Returns
        - The instance
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.config.ScriptOptionsConfig

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot import PyCore
from dev.magicmq.pyspigot.config import *
from dev.magicmq.pyspigot.exception import InvalidConfigurationException
from java.io import File
from java.io import FileInputStream
from java.io import IOException
from java.io import InputStream
from org.yaml.snakeyaml import DumperOptions
from org.yaml.snakeyaml import LoaderOptions
from org.yaml.snakeyaml import Yaml
from org.yaml.snakeyaml.constructor import SafeConstructor
from org.yaml.snakeyaml.nodes import Tag
from org.yaml.snakeyaml.representer import Representer
from org.yaml.snakeyaml.resolver import Resolver
from typing import Any, Callable, Iterable, Tuple


class ScriptOptionsConfig:
    """
    Helper class to retrieve configuration values from the script options config.
    """

    @staticmethod
    def reload() -> None:
        ...


    @staticmethod
    def contains(key: str) -> bool:
        ...


    @staticmethod
    def getScriptSection(scriptName: str) -> dict[Any, Any]:
        ...


    @staticmethod
    def getEnabled(scriptName: str, defaultValue: bool) -> bool:
        ...


    @staticmethod
    def getLoadPriority(scriptName: str, defaultValue: int) -> int:
        ...


    @staticmethod
    def getPluginDepend(scriptName: str, defaultValue: list[str]) -> list[str]:
        ...


    @staticmethod
    def getFileLoggingEnabled(scriptName: str, defaultValue: bool) -> bool:
        ...


    @staticmethod
    def getMinLoggingLevel(scriptName: str, defaultValue: str) -> str:
        ...


    @staticmethod
    def getPermissionDefault(scriptName: str, defaultValue: str) -> str:
        ...


    @staticmethod
    def getPermissions(scriptName: str, defaultValue: dict[Any, Any]) -> dict[Any, Any]:
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.redis.RedisManager

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.manager.redis import *
from dev.magicmq.pyspigot.manager.redis.client import RedisCommandClient
from dev.magicmq.pyspigot.manager.redis.client import RedisPubSubClient
from dev.magicmq.pyspigot.manager.redis.client import ScriptRedisClient
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.util import ScriptUtils
from io.lettuce.core import ClientOptions
from io.lettuce.core import RedisConnectionException
from io.lettuce.core import RedisURI
from java.util.concurrent import CompletableFuture
from typing import Any, Callable, Iterable, Tuple


class RedisManager:
    """
    Manager to interface with remote redis servers. Used by scripts to subscribe to pub/sub messaging and publish messages.
    """

    def newRedisURI(self) -> "RedisURI.Builder":
        """
        Get a new RedisURI builder for use when opening a new script redis client.

        Returns
        - A io.lettuce.core.RedisURI.Builder object used to build a URI with connection information
        """
        ...


    def newClientOptions(self) -> "ClientOptions.Builder":
        """
        Get a new client options builder for use when opening a new script redis client.

        Returns
        - A io.lettuce.core.ClientOptions.Builder object used to build ClientOptions for the RedisClient
        """
        ...


    def openRedisClient(self, clientType: "ClientType", ip: str, port: str, password: str) -> "ScriptRedisClient":
        """
        Initialize a new RedisPubSubClient with a connection to a remote redis server with the specified ip, port, and password, using the default client options. The connection to the remote redis server will be opened automatically when the client is created.
        
        **Note:** This should be called from scripts only!

        Arguments
        - clientType: The type of redis client to open, such as pub/sub or command
        - ip: The IP of the redis server to connect to
        - port: The port of the redis server to connect to
        - password: The password for the redis server

        Returns
        - A ScriptRedisClient representing a client that is connected to the remote redis server
        """
        ...


    def openRedisClient(self, clientType: "ClientType", ip: str, port: str, password: str, clientOptions: "ClientOptions") -> "ScriptRedisClient":
        """
        Initialize a new RedisPubSubClient with a connection to a remote redis server with the specified ip, port, and password, using the specified io.lettuce.core.ClientOptions. The connection to the remote redis server will be opened automatically when the client is created.
        
        **Note:** This should be called from scripts only!

        Arguments
        - clientType: The type of redis client to open, such as pub/sub or command
        - ip: The IP of the redis server to connect to
        - port: The port of the redis server to connect to
        - password: The password for the redis server
        - clientOptions: The io.lettuce.core.ClientOptions that should be used for the io.lettuce.core.RedisClient

        Returns
        - A ScriptRedisClient representing a client that is connected to the remote redis server
        """
        ...


    def openRedisClient(self, clientType: "ClientType", redisURI: "RedisURI") -> "ScriptRedisClient":
        """
        Initialize a new RedisPubSubClient with a connection to a remote redis server with the specified io.lettuce.core.RedisURI. The connection to the remote redis server will be opened automatically when the client is created.
        
        **Note:** This should be called from scripts only!

        Arguments
        - clientType: The type of redis client to open, such as pub/sub or command
        - redisURI: The URI specifying the connection details to the redis server

        Returns
        - A ScriptRedisClient representing a client that is connected to the remote redis server
        """
        ...


    def openRedisClient(self, clientType: "ClientType", redisURI: "RedisURI", clientOptions: "ClientOptions") -> "ScriptRedisClient":
        """
        Initialize a new RedisPubSubClient with a connection to a remote redis server with the specified io.lettuce.core.RedisURI and io.lettuce.core.ClientOptions. The connection to the remote redis server will be opened automatically when the client is created.
        
        **Note:** This should be called from scripts only!

        Arguments
        - clientType: The type of redis client to open, such as pub/sub or command
        - redisURI: The URI specifying the connection details to the redis server
        - clientOptions: The ClientOptions that should be used for the io.lettuce.core.RedisClient

        Returns
        - A ScriptRedisClient representing a client that is connected to the remote redis server
        """
        ...


    def closeRedisClient(self, client: "ScriptRedisClient") -> None:
        """
        Close the specified ScriptRedisClient synchronously. This method will block until the redis client is closed.
        
        **Note:** This should be called from scripts only!

        Arguments
        - client: The client to close
        """
        ...


    def closeRedisClientAsync(self, client: "ScriptRedisClient") -> "CompletableFuture"["Void"]:
        """
        Close the specified ScriptRedisClient asynchronously, without blocking.
        
        **Note:** This should be called from scripts only!

        Arguments
        - client: The client to close

        Returns
        - A CompletableFuture that completes when closing the client is finished
        """
        ...


    def closeRedisClients(self, script: "Script", async: bool) -> None:
        """
        Close all ScriptRedisClients belonging to a script.

        Arguments
        - script: The script whose ScriptRedisClients should be closed
        - async: Whether the client should be closed asynchronously
        """
        ...


    def getRedisClients(self, script: "Script") -> list["ScriptRedisClient"]:
        """
        Get all open ScriptRedisClients belonging to a script.

        Arguments
        - script: The script to get ScriptRedisClients from

        Returns
        - A List of ScriptRedisClient containing all clients associated with the script. Will return an empty list if there are no clients associated with the script
        """
        ...


    def getRedisClients(self, script: "Script", type: "ClientType") -> list["ScriptRedisClient"]:
        ...


    @staticmethod
    def get() -> "RedisManager":
        """
        Get the singleton instance of this RedisManager.

        Returns
        - The instance
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.manager.placeholder.ScriptPlaceholder

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit.manager.placeholder import *
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.manager.script import ScriptManager
from me.clip.placeholderapi.expansion import PlaceholderExpansion
from me.clip.placeholderapi.expansion import Relational
from org.bukkit import OfflinePlayer
from org.bukkit.entity import Player
from org.python.core import Py
from org.python.core import PyException
from org.python.core import PyFunction
from org.python.core import PyObject
from org.python.core import PyString
from typing import Any, Callable, Iterable, Tuple


class ScriptPlaceholder(PlaceholderExpansion, Relational):
    """
    A class that represents a script placeholder expansion.
    
    A ScriptPlaceholder can have multiple individual placeholders. For example, a script with the name "test.py" could have "%script:test_placeholder1%" and "%script:test_placeholder2%". It will be up to the script to handle each individual placeholder.

    See
    - me.clip.placeholderapi.expansion.PlaceholderExpansion
    """

    def __init__(self, script: "Script", function: Callable, relFunction: Callable, author: str, version: str):
        """
        Arguments
        - script: The script associated with this ScriptPlaceholder
        - function: The function to call when the placeholder is used
        - relFunction: The function to call when the relational placeholder is used
        - author: The author of this ScriptPlaceholder
        - version: The version of this ScriptPlaceholder
        """
        ...


    def getScript(self) -> "Script":
        """
        Get the script associated with this ScriptPlaceholder.

        Returns
        - The script associated with this ScriptPlaceholder
        """
        ...


    def getAuthor(self) -> str:
        """
        Get the author of this ScriptPlaceholder.

        Returns
        - The author of this ScriptPlaceholder
        """
        ...


    def getVersion(self) -> str:
        """
        Get the version of this ScriptPlaceholder.

        Returns
        - The version of this ScriptPlaceholder
        """
        ...


    def getIdentifier(self) -> str:
        """
        Get the identifier of this ScriptPlaceholder.
        
        This is used to identify the script's placeholder. It will be in the format "script:name", where "name" is the name of the script (without the file extension, .py). For example, for a script named "test.py", the placeholder identifier will be "script:test".

        Returns
        - The identifier of this ScriptPlaceholder
        """
        ...


    def persist(self) -> bool:
        """
        Indicates that the ScriptPlaceholder should persist when PlaceholderAPI is reloaded.

        Returns
        - True
        """
        ...


    def onRequest(self, player: "OfflinePlayer", params: str) -> str:
        """
        Called internally when the ScriptPlaceholder is used.

        Arguments
        - player: The org.bukkit.OfflinePlayer associated with the placeholder, or null if there is none
        - params: The specific placeholder that was used (the ScriptPlaceholder expansion can have multiple individual placeholders. Scripts will handle each specific placeholder on their own)

        Returns
        - The replaced text
        """
        ...


    def onPlaceholderRequest(self, playerOne: "Player", playerTwo: "Player", identifier: str) -> str:
        """
        Called internally when the ScriptPlaceholder is used in a relational fashion.

        Arguments
        - playerOne: The first org.bukkit.entity.Player used for the placeholder.
        - playerTwo: The second org.bukkit.entity.Player used for the placeholder.
        - identifier: The specific placeholder that was used, right after the relational aspect.

        Returns
        - The replaced text
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.task.Task

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.manager.script import ScriptManager
from dev.magicmq.pyspigot.manager.task import *
from java.util import Arrays
from org.python.core import Py
from org.python.core import PyBaseCode
from org.python.core import PyException
from org.python.core import PyFunction
from org.python.core import PyObject
from typing import Any, Callable, Iterable, Tuple


class Task(Runnable):
    """
    Represents a task defined by a script.
    """

    def __init__(self, script: "Script", function: Callable, functionArgs: list["Object"], async: bool, delay: int):
        """
        Arguments
        - script: The script associated with this task
        - function: The script function that should be called when the task executes
        - functionArgs: Any arguments that should be passed to the function
        - async: True if the task is asynchronous, False if otherwise
        - delay: The delay, in ticks, to wait until running the task
        """
        ...


    def run(self) -> None:
        """
        Called internally when the task executes.
        """
        ...


    def getScript(self) -> "Script":
        """
        Get the script associated with this task.

        Returns
        - The script associated with this task
        """
        ...


    def getTaskId(self) -> int:
        """
        Get the ID for this task.

        Returns
        - The task ID for this task
        """
        ...


    def setTaskId(self, taskId: int) -> None:
        """
        Set the task ID for this task.

        Arguments
        - taskId: The task ID to set
        """
        ...


    def toString(self) -> str:
        """
        Prints a representation of this Task in string format, including the task ID, if it is async, and delay (if applicable)

        Returns
        - A string representation of the Task
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.script.ScriptOptions

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot import PyCore
from dev.magicmq.pyspigot.config import ScriptOptionsConfig
from dev.magicmq.pyspigot.exception import InvalidConfigurationException
from dev.magicmq.pyspigot.manager.script import *
from typing import Any, Callable, Iterable, Tuple


class ScriptOptions:
    """
    A class representing various runtime options belonging to a certain script.
    """

    def __init__(self):
        """
        Initialize a new ScriptOptions with the default values.
        """
        ...


    def __init__(self, scriptName: str):
        """
        Initialize a new ScriptOptions using the appropriate values in the script_options.yml file, using the script name to search for the values.

        Arguments
        - scriptName: The name of the script whose script options should be initialized
        """
        ...


    def isEnabled(self) -> bool:
        """
        Get if this script is enabled.

        Returns
        - True if the script is enabled, False if otherwise
        """
        ...


    def getLoadPriority(self) -> int:
        """
        Get the load priority for this script. Scripts with greater load priority will load before scripts with lower load priority.

        Returns
        - The script's load priority
        """
        ...


    def getPluginDependencies(self) -> list[str]:
        """
        Get a list of plugin dependencies for this script.

        Returns
        - A list of plugin dependencies for this script. Will return an empty list if this script has no plugin dependencies
        """
        ...


    def isFileLoggingEnabled(self) -> bool:
        """
        Get if file logging is enabled for this script.

        Returns
        - True if file logging is enabled, False if otherwise
        """
        ...


    def getMinLoggingLevel(self) -> "Level":
        """
        Get the minimum logging level for this script, represented as a java.util.logging.Level

        Returns
        - The minimum logging level at which messages should be logged
        """
        ...


    def toString(self) -> str:
        """
        Prints a representation of this ScriptOptions in string format, including all options as defined in script_options.yml

        Returns
        - A string representation of the ScriptOptions
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.manager.script.BukkitScriptOptions

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot import PyCore
from dev.magicmq.pyspigot.bukkit.manager.script import *
from dev.magicmq.pyspigot.config import ScriptOptionsConfig
from dev.magicmq.pyspigot.exception import InvalidConfigurationException
from dev.magicmq.pyspigot.manager.script import ScriptOptions
from org.bukkit.permissions import Permission
from org.bukkit.permissions import PermissionDefault
from typing import Any, Callable, Iterable, Tuple


class BukkitScriptOptions(ScriptOptions):
    """
    An extension of the base ScriptOptions class that includes Bukkit-specific code for parsing and registering script permissions.
    """

    def __init__(self):
        """
        Initialize a new ScriptOptions with the default values.
        """
        ...


    def __init__(self, scriptName: str):
        """
        Initialize a new ScriptOptions using the appropriate values in the script_options.yml file, using the script name to search for the values.

        Arguments
        - scriptName: The name of the script whose script options should be initialized
        """
        ...


    def getPermissionDefault(self) -> "PermissionDefault":
        """
        Get the default permissions for permissions defined for this script.

        Returns
        - The default permission level
        """
        ...


    def getPermissions(self) -> list["Permission"]:
        """
        Get a list of permissions defined for this script.

        Returns
        - A list of permissions. Will return an empty list if this script has no permissions defined
        """
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.util.player.CommandSenderAdapter

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.util.player import *
from net.md_5.bungee.api.chat import BaseComponent
from typing import Any, Callable, Iterable, Tuple


class CommandSenderAdapter:
    """
    A utility class that wraps a platform-specific command sender object.
    """

    def hasPermission(self, permission: str) -> bool:
        """
        Check if the command sender has a permission via a platform-specific implementation.

        Arguments
        - permission: The permission to check

        Returns
        - True if the command sender has the permission, False if it does not
        """
        ...


    def sendMessage(self, message: str) -> None:
        """
        Send a message to the command sender via a platform-specific implementation.

        Arguments
        - message: The message to send
        """
        ...


    def sendMessage(self, message: list["BaseComponent"]) -> None:
        """
        Send a message to the command sender via a platform-specific implementation.

        Arguments
        - message: The message to send
        """
        ...


    def isPlayer(self) -> bool:
        """
        Check if the command sender is a player via a platform-specific implementation.

        Returns
        - True if the command sender is a player, False if it is not
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.command.CommandManager

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.manager.command import *
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.util import ScriptUtils
from org.python.core import PyFunction
from typing import Any, Callable, Iterable, Tuple


class CommandManager:
    """
    Abstract manager to interface with a server-specific command framework. Primarily used by scripts to register and unregister commands.
    """

    def registerCommand(self, commandFunction: Callable, name: str) -> "ScriptCommand":
        """
        Register a new command.
        
        **Note:** This should be called from scripts only!

        Arguments
        - commandFunction: The command function that should be called when the command is executed
        - name: The name of the command to register

        Returns
        - A ScriptCommand representing the command that was registered
        """
        ...


    def registerCommand(self, commandFunction: Callable, tabFunction: Callable, name: str) -> "ScriptCommand":
        """
        Register a new command.
        
        **Note:** This should be called from scripts only!

        Arguments
        - commandFunction: The command function that should be called when the command is executed
        - tabFunction: The tab function that should be called for tab completion of the command
        - name: The name of the command to register

        Returns
        - A ScriptCommand representing the command that was registered
        """
        ...


    def registerCommand(self, commandFunction: Callable, name: str, permission: str) -> "ScriptCommand":
        """
        Register a new command.
        
        **Note:** This should be called from scripts only!

        Arguments
        - commandFunction: The command function that should be called when the command is executed
        - name: The name of the command to register
        - permission: The required permission node to use this command. Can be null

        Returns
        - A ScriptCommand representing the command that was registered
        """
        ...


    def registerCommand(self, commandFunction: Callable, tabFunction: Callable, name: str, permission: str) -> "ScriptCommand":
        """
        Register a new command.
        
        **Note:** This should be called from scripts only!

        Arguments
        - commandFunction: The command function that should be called when the command is executed
        - tabFunction: The tab function that should be called for tab completion of the command
        - name: The name of the command to register
        - permission: The required permission node to use this command. Can be null

        Returns
        - A ScriptCommand representing the command that was registered
        """
        ...


    def registerCommand(self, commandFunction: Callable, name: str, aliases: list[str], permission: str) -> "ScriptCommand":
        """
        Register a new command.
        
        **Note:** This should be called from scripts only!

        Arguments
        - commandFunction: The command function that should be called when the command is executed
        - name: The name of the command to register
        - aliases: A List of String containing all the aliases for this command. Use an empty list for no aliases
        - permission: The required permission node to use this command. Can be null

        Returns
        - A ScriptCommand representing the command that was registered
        """
        ...


    def registerCommand(self, commandFunction: Callable, tabFunction: Callable, name: str, aliases: list[str], permission: str) -> "ScriptCommand":
        """
        Register a new command.
        
        **Note:** This should be called from scripts only!

        Arguments
        - commandFunction: The command function that should be called when the command is executed
        - tabFunction: The tab function that should be called for tab completion of the command
        - name: The name of the command to register
        - aliases: A List of String containing all the aliases for this command. Use an empty list for no aliases
        - permission: The required permission node to use this command. Can be null

        Returns
        - A ScriptCommand representing the command that was registered
        """
        ...


    def registerCommand(self, commandFunction: Callable, name: str, description: str, usage: str) -> "ScriptCommand":
        """
        Register a new command.
        
        **Note:** This should be called from scripts only!

        Arguments
        - commandFunction: The command function that should be called when the command is executed
        - name: The name of the command to register
        - description: The description of the command. Can be null (or an empty string)
        - usage: The usage message for the command. Can be null (or an empty string)

        Returns
        - A ScriptCommand representing the command that was registered
        """
        ...


    def registerCommand(self, commandFunction: Callable, tabFunction: Callable, name: str, description: str, usage: str) -> "ScriptCommand":
        """
        Register a new command.
        
        **Note:** This should be called from scripts only!

        Arguments
        - commandFunction: The command function that should be called when the command is executed
        - tabFunction: The tab function that should be called for tab completion of the command
        - name: The name of the command to register
        - description: The description of the command. Can be null (or an empty string)
        - usage: The usage message for the command. Can be null (or an empty string)

        Returns
        - A ScriptCommand representing the command that was registered
        """
        ...


    def registerCommand(self, commandFunction: Callable, name: str, description: str, usage: str, aliases: list[str]) -> "ScriptCommand":
        """
        Register a new command.
        
        **Note:** This should be called from scripts only!

        Arguments
        - commandFunction: The command function that should be called when the command is executed
        - name: The name of the command to register
        - description: The description of the command. Can be null (or an empty string)
        - usage: The usage message for the command. Can be null (or an empty string)
        - aliases: A List of String containing all the aliases for this command. Use an empty list for no aliases

        Returns
        - A ScriptCommand representing the command that was registered
        """
        ...


    def registerCommand(self, commandFunction: Callable, tabFunction: Callable, name: str, description: str, usage: str, aliases: list[str]) -> "ScriptCommand":
        """
        Register a new command.
        
        **Note:** This should be called from scripts only!

        Arguments
        - commandFunction: The command function that should be called when the command is executed
        - tabFunction: The tab function that should be called for tab completion of the command
        - name: The name of the command to register
        - description: The description of the command. Can be null (or an empty string)
        - usage: The usage message for the command. Can be null (or an empty string)
        - aliases: A List of String containing all the aliases for this command. Use an empty list for no aliases

        Returns
        - A ScriptCommand representing the command that was registered
        """
        ...


    def registerCommand(self, commandFunction: Callable, tabFunction: Callable, name: str, description: str, usage: str, aliases: list[str], permission: str) -> "ScriptCommand":
        """
        Register a new command.
        
        **Note:** This should be called from scripts only!

        Arguments
        - commandFunction: The command function that should be called when the command is executed
        - tabFunction: The tab function that should be called for tab completion of the command. Can be null
        - name: The name of the command to register
        - description: The description of the command. Can be null (or an empty string)
        - usage: The usage message for the command. Can be null (or an empty string)
        - aliases: A List of String containing all the aliases for this command. Use an empty list for no aliases
        - permission: The required permission node to use this command. Can be null

        Returns
        - A ScriptCommand representing the command that was registered
        """
        ...


    def unregisterCommand(self, command: "ScriptCommand") -> None:
        """
        Unregister a script's command.
        
        **Note:** This should be called from scripts only!

        Arguments
        - command: The command to be unregistered
        """
        ...


    def unregisterCommands(self, script: "Script") -> None:
        """
        Unregister all commands belonging to a particular script.

        Arguments
        - script: The script from which all commands should be unregistered
        """
        ...


    def getCommand(self, script: "Script", name: str) -> "ScriptCommand":
        """
        Get a command associated with a particular script by the command name

        Arguments
        - script: The script
        - name: The name of the command

        Returns
        - The command with this name and associated with the script, or null if none was found
        """
        ...


    def getCommands(self, script: "Script") -> list["ScriptCommand"]:
        """
        Get an immutable list containing all commands belonging to a particular script.

        Arguments
        - script: The script to get commands from

        Returns
        - An immutable list containing all commands belonging to the script. Will return null if no commands belong to the script
        """
        ...


    @staticmethod
    def get() -> "CommandManager":
        """
        Get the singleton instance of this CommandManager.

        Returns
        - The instance
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.util.CommandAliasHelpTopic

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit.util import *
from org.bukkit import ChatColor
from org.bukkit.command import CommandSender
from org.bukkit.help import HelpMap
from org.bukkit.help import HelpTopic
from typing import Any, Callable, Iterable, Tuple


class CommandAliasHelpTopic(HelpTopic):
    """
    Represents a help topic for an alias of a command.
    
    Copied from org.bukkit.craftbukkit.help.CommandAliasHelpTopic
    """

    def __init__(self, alias: str, aliasFor: str, helpMap: "HelpMap"):
        ...


    def getFullText(self, forWho: "CommandSender") -> str:
        ...


    def canSee(self, commandSender: "CommandSender") -> bool:
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.manager.listener.BukkitScriptEventExecutor

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit.event import ScriptExceptionEvent
from dev.magicmq.pyspigot.bukkit.manager.listener import *
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.manager.script import ScriptManager
from org.bukkit.event import Event
from org.bukkit.event import Listener
from org.bukkit.plugin import EventExecutor
from org.python.core import Py
from org.python.core import PyException
from org.python.core import PyObject
from typing import Any, Callable, Iterable, Tuple


class BukkitScriptEventExecutor(EventExecutor):
    """
    Represents a Bukkit event executor for script event listeners.

    See
    - org.bukkit.plugin.EventExecutor
    """

    def __init__(self, scriptEventListener: "BukkitScriptEventListener", eventClass: type["Event"]):
        """
        Arguments
        - scriptEventListener: The BukkitScriptEventListener associated with this ScriptEventExecutor
        - eventClass: The Bukkit event associated with this ScriptEventExecutor. Should be a Class of the Bukkit event
        """
        ...


    def execute(self, listener: "Listener", event: "Event") -> None:
        """
        Called internally when the event occurs.

        Arguments
        - listener: The listener associated with this EventExecutor
        - event: The event that occurred
        """
        ...



"""
This is a helper module that wraps a variety of functional interfaces in the java.util.function package, for easier use in Python.

This module should be particularly useful when working with some libraries that make use of functional interfaces (such as NBT-API).
"""
import java.util.function.BiConsumer
import java.util.function.BiFunction
import java.util.function.BiPredicate
import java.util.function.Consumer
import java.util.function.Function
import java.util.function.Predicate
import java.util.function.Supplier

"""
Represents an operation that accepts two input arguments and returns no result. This is the two-arity specialization of Consumer. Unlike most other functional interfaces, BiConsumer is expected to operate via side-effects.
"""
class BiConsumer(java.util.function.BiConsumer):

    """
    Initialize a new BiConsumer.

    Arguments:
        function (callable): A function that accepts two arguments and returns nothing.
    """
    def __init__(self, function):
        self.accept = function


"""
Represents a function that accepts two arguments and produces a result. This is the two-arity specialization of Function. 
"""
class BiFunction(java.util.function.BiFunction):

    """
    Initializes a new BiFunction.

    Arguments:
        function (callable): A function that accepts two arguments and returns a value.
    """
    def __init__(self, function):
        self.apply = function


"""
Represents a predicate (boolean-valued function) of two arguments. This is the two-arity specialization of Predicate. 
"""
class BiPredicate(java.util.function.BiPredicate):

    """
    Initializes a new BiPredicate.

    Arguments:
        function (callable): A function that accepts two arguments and returns either True or False.
    """
    def __init__(self, function):
        self.test = function


"""
Represents an operation that accepts a single input argument and returns no result. Unlike most other functional interfaces, Consumer is expected to operate via side-effects.
"""
class Consumer(java.util.function.Consumer):

    """
    Initializes a new Consumer.

    Arguments:
        function (callable): A function that accepts one argument and returns nothing.
    """
    def __init__(self, function):
        self.accept = function


"""
Represents a function that accepts one argument and produces a result.
"""
class Function(java.util.function.Function):

    """
    Initializes a new Function.

    Arguments:
        function (callable): A function that accepts one argument and returns a value.
    """
    def __init__(self, function):
        self.apply = function


"""
Represents a predicate (boolean-valued function) of one argument.
"""
class Predicate(java.util.function.Predicate):

    """
    Initializes a new Predicate.

    Arguments:
        function (callable): A function that accepts one argument and returns either True or False.
    """
    def __init__(self, function):
        self.test = function


"""
Represents a supplier of results.

There is no requirement that a new or distinct result be returned each time the supplier is invoked.
"""
class Supplier(java.util.function.Supplier):

    """
    Initializes a new Supplier.

    Arguments:
        function (callable): A function that accepts no arguments and returns a value.
    """
    def __init__(self, function):
        self.get = function


"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.event.ScriptExceptionEvent

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit.event import *
from dev.magicmq.pyspigot.manager.script import Script
from org.bukkit.event import HandlerList
from org.python.core import PyException
from typing import Any, Callable, Iterable, Tuple


class ScriptExceptionEvent(ScriptEvent):
    """
    Called when a script throws an unhandled error/exception. This event could be called asynchronously if the exception occurred in an asynchronous context. To check if the event is asynchronous, call org.bukkit.event.Event.isAsynchronous()
    
    The exception will be a org.python.core.PyException, which will include Java exceptions thrown by calls to Java code from scripts. Use org.python.core.PyException.getCause to determine if there was an underlying Java exception.
    """

    def __init__(self, script: "Script", exception: "PyException", async: bool):
        """
        Arguments
        - script: The script that caused the error/exception
        - exception: The org.python.core.PyException that was thrown
        - async: Whether the exception occurred in an asychronous context
        """
        ...


    def getException(self) -> "PyException":
        """
        Get the org.python.core.PyException that was thrown.

        Returns
        - The org.python.core.PyException that was thrown
        """
        ...


    def doReportException(self) -> bool:
        """
        Get if the exception should be reported to console and/or a script's log file.

        Returns
        - True if the exception should be reported to console and/or a script's log file, False if otherwise
        """
        ...


    def setReportException(self, reportException: bool) -> None:
        """
        Set if the exception should be reported to console and/or the script's log file.

        Arguments
        - reportException: Whether the exception should be reported.
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.BukkitListener

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot import PluginListener
from dev.magicmq.pyspigot import PyCore
from dev.magicmq.pyspigot.bukkit import *
from dev.magicmq.pyspigot.bukkit.util.player import BukkitPlayer
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.manager.script import ScriptManager
from dev.magicmq.pyspigot.util.player import PlayerAdapter
from org.bukkit import Bukkit
from org.bukkit.event import EventHandler
from org.bukkit.event import Listener
from org.bukkit.event.player import PlayerJoinEvent
from org.bukkit.event.server import PluginDisableEvent
from typing import Any, Callable, Iterable, Tuple


class BukkitListener(PluginListener, Listener):
    """
    The Bukkit listener.
    """

    def onDisable(self, event: "PluginDisableEvent") -> None:
        ...


    def onJoin(self, event: "PlayerJoinEvent") -> None:
        ...



from .ScriptOptions import ScriptOptions
from .ScriptManager import ScriptManager
from .ScriptInfo import ScriptInfo
from .Script import Script
from .RunResult import RunResult
from .GlobalVariables import GlobalVariables


"""
Python module generated from Java source file dev.magicmq.pyspigot.bukkit.manager.script.BukkitScriptInfo

Java source file obtained from artifact pyspigot-bukkit version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.bukkit import PySpigot
from dev.magicmq.pyspigot.bukkit.manager.placeholder import PlaceholderManager
from dev.magicmq.pyspigot.bukkit.manager.placeholder import ScriptPlaceholder
from dev.magicmq.pyspigot.bukkit.manager.protocol import ProtocolManager
from dev.magicmq.pyspigot.bukkit.manager.protocol import ScriptPacketListener
from dev.magicmq.pyspigot.bukkit.manager.script import *
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.manager.script import ScriptInfo
from net.md_5.bungee.api import ChatColor
from typing import Any, Callable, Iterable, Tuple


class BukkitScriptInfo(ScriptInfo):
    """
    The Bukkit-specific implementation of the dev.magicmq.pyspigot.manager.script.ScriptInfo class, for printing information related to Bukkit-specific managers.
    """

    def printPlatformManagerInfo(self, script: "Script", appendTo: "StringBuilder") -> None:
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.redis.client.ScriptRedisClient

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot import PyCore
from dev.magicmq.pyspigot.manager.redis.client import *
from dev.magicmq.pyspigot.manager.script import Script
from io.lettuce.core import ClientOptions
from io.lettuce.core import RedisClient
from io.lettuce.core import RedisURI
from java.util.concurrent import CompletableFuture
from typing import Any, Callable, Iterable, Tuple


class ScriptRedisClient:
    """
    A wrapper class that wraps the RedisClient from lettuce for use by scripts.

    See
    - io.lettuce.core.RedisClient
    """

    def __init__(self, script: "Script", redisURI: "RedisURI", clientOptions: "ClientOptions"):
        """
        Arguments
        - script: The script to which this ScriptRedisClient belongs
        - redisURI: The URI that specifies the connection details to the server
        - clientOptions: The io.lettuce.core.ClientOptions that should be used for the RedisClient
        """
        ...


    def open(self) -> None:
        """
        Initialize a new io.lettuce.core.RedisClient and open a connection to the remote redis server.
        """
        ...


    def close(self) -> None:
        """
        Close the open to the remote redis server synchronously, blocking if necessary.
        """
        ...


    def closeAsync(self) -> "CompletableFuture"["Void"]:
        """
        Close the open connection to the remote redis server asynchronously.

        Returns
        - A CompletableFuture that completes when the shutdown is finished
        """
        ...


    def getScript(self) -> "Script":
        """
        Get the script associated with this redis client.

        Returns
        - The script associated with this redis client.
        """
        ...


    def getClientId(self) -> int:
        """
        Get the ID of this redis client.

        Returns
        - The ID
        """
        ...


    def getRedisURI(self) -> "RedisURI":
        """
        Get the io.lettuce.core.RedisURI of this redis client.

        Returns
        - The RedisURI
        """
        ...


    def getClientOptions(self) -> "ClientOptions":
        """
        Get the io.lettuce.core.ClientOptions of this redis client.

        Returns
        - The ClientOptions
        """
        ...


    def getRedisClient(self) -> "RedisClient":
        """
        Get the underlying lettuce io.lettuce.core.RedisClient for this ScriptRedisClient.

        Returns
        - The RedisClient associated with this ScriptRedisClient
        """
        ...


    def toString(self) -> str:
        """
        Prints a representation of this ScriptRedisClient in string format

        Returns
        - A string representation of the ScriptRedisClient
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.task.TaskManager

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.manager.script import Script
from dev.magicmq.pyspigot.manager.task import *
from dev.magicmq.pyspigot.util import ScriptUtils
from org.python.core import PyFunction
from typing import Any, Callable, Iterable, Tuple


class TaskManager:
    """
    Manager to interface with a server platform's scheduler. Primarily used by scripts to register and unregister tasks.
    """

    def runTask(self, function: Callable, *functionArgs: Tuple["Object", ...]) -> int:
        """
        Schedule a new synchronous task via a platform-specific implementation.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the synchronous task executes
        - functionArgs: Any arguments that should be passed to the function

        Returns
        - An ID representing the synchronous task that was scheduled
        """
        ...


    def runTaskAsync(self, function: Callable, *functionArgs: Tuple["Object", ...]) -> int:
        """
        Schedule a new asynchronous task via a platform-specific implementation.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the asynchronous task executes
        - functionArgs: Any arguments that should be passed to the function

        Returns
        - An ID representing the asynchronous task that was scheduled
        """
        ...


    def runTaskLater(self, function: Callable, delay: int, *functionArgs: Tuple["Object", ...]) -> int:
        """
        Schedule a new synchronous task to run at a later point in time via a platform-specific implementation.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the synchronous task executes
        - delay: The delay, in ticks, that the scheduler should wait before executing the synchronous task
        - functionArgs: Any arguments that should be passed to the function

        Returns
        - An ID representing the synchronous task that was scheduled
        """
        ...


    def runTaskLaterAsync(self, function: Callable, delay: int, *functionArgs: Tuple["Object", ...]) -> int:
        """
        Schedule a new asynchronous task to run at a later point in time via a platform-specific implementation.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the asynchronous task executes
        - delay: The delay, in ticks, that the scheduler should wait before executing the asynchronous task
        - functionArgs: Any arguments that should be passed to the function

        Returns
        - An ID representing the asynchronous task that was scheduled
        """
        ...


    def scheduleRepeatingTask(self, function: Callable, delay: int, interval: int, *functionArgs: Tuple["Object", ...]) -> int:
        """
        Schedule a new synchronous repeating task via a platform-specific implementation.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called each time the synchronous task executes
        - delay: The delay, in ticks, to wait before beginning this synchronous repeating task
        - interval: The interval, in ticks, that the synchronous repeating task should be executed
        - functionArgs: Any arguments that should be passed to the function

        Returns
        - An ID representing the synchronous task that was scheduled
        """
        ...


    def scheduleAsyncRepeatingTask(self, function: Callable, delay: int, interval: int, *functionArgs: Tuple["Object", ...]) -> int:
        """
        Schedule a new asynchronous repeating task via a platform-specific implementation.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called each time the asynchronous task executes
        - delay: The delay, in ticks, to wait before beginning this asynchronous repeating task
        - interval: The interval, in ticks, that the asynchronous repeating task should be executed
        - functionArgs: Any arguments that should be passed to the function

        Returns
        - An ID representing the asynchronous task that was scheduled
        """
        ...


    def runSyncCallbackTask(self, function: Callable, callback: Callable, *functionArgs: Tuple["Object", ...]) -> int:
        """
        Schedule a new asynchronous task with a synchronous callback via a platform-specific implementation. Data returned from the initially called function (asynchronous portion) is automatically passed to the synchronous callback function as a function argument.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the asynchronous task executes
        - callback: The function that should be called for the synchronous callback once the asynchronous portion of the task finishes
        - functionArgs: Any arguments that should be passed to the function

        Returns
        - An ID representing the asynchronous task that was scheduled
        """
        ...


    def runSyncCallbackTaskLater(self, function: Callable, callback: Callable, delay: int, *functionArgs: Tuple["Object", ...]) -> int:
        """
        Schedule a new asynchronous task with a synchronous callback to run at a later point in time via a platform-specific implementation. Data returned from the initially called function (asynchronous portion) is automatically passed to the synchronous callback function as a function argument.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the asynchronous task executes
        - callback: The function that should be called for the synchronous callback once the asynchronous portion of the task finishes
        - delay: The delay, in ticks, that the scheduler should wait before executing the asynchronous task
        - functionArgs: Any arguments that should be passed to the function

        Returns
        - An ID representing the asynchronous task that was scheduled
        """
        ...


    def stopTask(self, taskId: int) -> None:
        """
        Terminate a task with the given task ID via a platform-specific implementation.

        Arguments
        - taskId: The ID of the task to terminate
        """
        ...


    def stopTasks(self, script: "Script") -> None:
        """
        Terminate all scheduled tasks belonging to a script via a platform-specific implementation.

        Arguments
        - script: The script whose scheduled tasks should be terminated
        """
        ...


    def getTask(self, taskId: int) -> "Task":
        """
        Get a scheduled task from its ID.

        Arguments
        - taskId: The task ID

        Returns
        - The scheduled task associated with the task ID, null if no task was found with the given ID
        """
        ...


    def getTasks(self, script: "Script") -> list["Task"]:
        """
        Get all scheduled tasks associated with a script.

        Arguments
        - script: The script whose scheduled tasks should be gotten

        Returns
        - An immutable list containing all scheduled tasks associated with the script. Returns null if the script has no scheduled tasks
        """
        ...


    @staticmethod
    def get() -> "TaskManager":
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.listener.ListenerManager

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.manager.listener import *
from dev.magicmq.pyspigot.manager.script import Script
from org.python.core import PyFunction
from typing import Any, Callable, Iterable, Tuple


class ListenerManager:
    """
    Abstract manager to interface with a server-specific event framework. Primarily used by scripts to register and unregister event listeners.
    
    Type `<T>`: The platform-specific Listener class
    
    Type `<S>`: The platform-specific Event class
    
    Type `<U>`: The platform-specific EventPriority class
    """

    def registerListener(self, function: Callable, eventClass: type["S"]) -> "T":
        """
        Register a new event listener with default priority.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the event occurs
        - eventClass: The type of event to listen to

        Returns
        - The ScriptEventListener that was registered
        """
        ...


    def registerListener(self, function: Callable, eventClass: type["S"], priority: "U") -> "T":
        """
        Register a new event listener.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the event occurs
        - eventClass: The type of event to listen to
        - priority: The priority of the event relative to other listeners

        Returns
        - The ScriptEventListener that was registered
        """
        ...


    def registerListener(self, function: Callable, eventClass: type["S"], ignoreCancelled: bool) -> "T":
        """
        Register a new event listener with default priority.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the event occurs
        - eventClass: The type of event to listen to
        - ignoreCancelled: If True, the event listener will not be called if the event has been previously cancelled by another listener.

        Returns
        - The ScriptEventListener that was registered
        """
        ...


    def registerListener(self, function: Callable, eventClass: type["S"], priority: "U", ignoreCancelled: bool) -> "T":
        """
        Register a new event listener.
        
        **Note:** This should be called from scripts only!

        Arguments
        - function: The function that should be called when the event occurs
        - eventClass: The type of event to listen to
        - priority: The priority of the event relative to other listeners
        - ignoreCancelled: If True, the event listener will not be called if the event has been previously cancelled by another listener.

        Returns
        - The ScriptEventListener that was registered
        """
        ...


    def unregisterListener(self, listener: "T") -> None:
        """
        Unregister an event listener.
        
        **Note:** This should be called from scripts only!

        Arguments
        - listener: The listener to unregister
        """
        ...


    def unregisterListeners(self, script: "Script") -> None:
        """
        Unregister all event listeners belonging to a script.

        Arguments
        - script: The script whose event listeners should be unregistered
        """
        ...


    def getListener(self, script: "Script", eventClass: type["S"]) -> "T":
        """
        Get the event listener for a particular event associated with a script

        Arguments
        - script: The script
        - eventClass: The event

        Returns
        - The listener associated with the script and event, or null if there is none
        """
        ...


    def getListeners(self, script: "Script") -> list["T"]:
        """
        Get all event listeners associated with a script

        Arguments
        - script: The script to get event listeners from

        Returns
        - An immutable List of listeners containing all event listeners associated with the script. Will return null if there are no event listeners associated with the script
        """
        ...


    @staticmethod
    def get() -> "ListenerManager"[Any, Any, Any]:
        """
        Get the singleton instance of this ListenerManager.

        Returns
        - The instance
        """
        ...



"""
Python module generated from Java source file dev.magicmq.pyspigot.manager.database.DatabaseType

Java source file obtained from artifact pyspigot-core version 0.8.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from dev.magicmq.pyspigot.manager.database import *
from dev.magicmq.pyspigot.manager.database.mongo import MongoDatabase
from dev.magicmq.pyspigot.manager.database.sql import SqlDatabase
from enum import Enum
from typing import Any, Callable, Iterable, Tuple


class DatabaseType(Enum):
    """
    Utility enum to represent different database types available for scripts to use.
    """

    SQL = (SqlDatabase, /*Host, port, database, user, password*/
    "jdbc:mysql://%s:%s/%s?user=%s&password=%s")
    """
    An SQL database type.
    """
    MONGO_DB = (MongoDatabase, /*User, password, host, port*/
    "mongodb://%s:%s@%s:%s")
    """
    A MongoDB database type.
    """
    MONGO_DB_NO_AUTH = (MongoDatabase, "mongodb://%s:%s")
    """
    A MongoDB database type without authentication.
    """


    def getDbClass(self) -> type["Database"]:
        """
        Get the class that pertains to the database type. Will be a subclass of Database

        Returns
        - The class associated with the database type
        """
        ...


    def getUri(self) -> str:
        """
        Get the URI scheme associated with the database type.

        Returns
        - The URI scheme associated with the database type
        """
        ...
